from numpy import column_stack, array, exp, real, squeeze, array2string
from SciDataTool.Functions.Plot.plot_2D import plot_2D
from SciDataTool.Functions.conversions import (
    rphiz_to_xyz,
    xyz_to_rphiz,
    rphiz_to_xyz_field,
)
from SciDataTool.GUI.WVectorSelector.WVectorSelector import REV_COMP_DICT
from SciDataTool.Functions.Load.import_class import import_class


def plot_2D_Data(
    self,
    *arg_list,
    axis_data=None,
    radius=None,
    is_norm=False,
    unit="SI",
    component_list=None,
    data_list=[],
    legend_list=[],
    color_list=None,
    linestyles=None,
    linewidth_list=[2],
    save_path=None,
    x_min=None,
    x_max=None,
    y_min=None,
    y_max=None,
    is_logscale_x=False,
    is_logscale_y=False,
    is_disp_title=True,
    is_grid=True,
    is_auto_ticks=True,
    is_auto_range=False,
    fig=None,
    ax=None,
    barwidth=100,
    type_plot=None,
    fund_harm_dict=None,
    is_show_fig=None,
    win_title=None,
    thresh=None,
    font_name="arial",
    font_size_title=12,
    font_size_label=10,
    font_size_legend=8,
    scale_units="x",
    scale=None,
    width=0.005,
    phase=0,
    is_show_legend=True,
    is_outside_legend=False,
    is_frame_legend=True,
):
    """Plots a field as a function of time

    Parameters
    ----------
    self : Output
        an Output object
    Data_str : str
        name of the Data Object to plot (e.g. "mag.Br")
    *arg_list : list of str
        arguments to specify which axes to plot
    radius : float
        radius used for quiver plot
    is_norm : bool
        boolean indicating if the field must be normalized
    unit : str
        unit in which to plot the field
    data_list : list
        list of Data objects to compare
    component_list : list
        list of component names to plot in separate figures
    legend_list : list
        list of legends to use for each Data object (including reference one) instead of data.name
    color_list : list
        list of colors to use for each Data object
    save_path : str
        full path of the png file where the figure is saved if save_path is not None
    x_min : float
        minimum value for the x-axis
    x_max : float
        maximum value for the x-axis
    y_min : float
        minimum value for the y-axis
    y_max : float
        maximum value for the y-axis
    is_logscale_x : bool
        boolean indicating if the x-axis must be set in logarithmic scale
    is_logscale_y : bool
        boolean indicating if the y-axis must be set in logarithmic scale
    is_disp_title : bool
        boolean indicating if the title must be displayed
    is_grid : bool
        boolean indicating if the grid must be displayed
    is_auto_ticks : bool
        in fft, adjust ticks to freqs (deactivate if too close)
    is_auto_range : bool
        in fft, display up to 1% of max
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        ax on which to plot the data
    barwidth : float
        barwidth scaling factor, only if type_plot = "bargraph"
    type_plot : str
        type of 2D graph : "curve", "bargraph", "barchart" or "quiver"
    fund_harm_dict : dict
        Dict containing axis name as key and frequency/order/wavenumber of fundamental harmonic as value to display fundamental harmonic in red in the fft
    is_show_fig : bool
        True to show figure after plot
    thresh : float
        threshold for automatic fft ticks
    scale_units : str
        arrow lenght scale factor reference {'width', 'height', 'dots', 'pouces', 'x', 'y', 'xy'}
    scale : float
        arrow length factor
    width : float
        arrow width factor
    is_outside_legend : bool
        True to display legend outside the graph
    is_frame_legend : bool
        True to display legend in a frame
    """

    # Dynamic import to avoid import loop
    DataPattern = import_class("SciDataTool.Classes", "DataPattern")

    # Special case of quiver plot
    if type_plot == "quiver":

        if component_list is None:
            title = "Total " + self.name
            result = self.get_xyz_along(
                arg_list,
                axis_data=axis_data,
                unit=unit,
                is_norm=is_norm,
            )

            if "x" in result and "y" in result:
                Xdatas = column_stack((result["x"], result["y"]))
            else:
                if radius is None:
                    radius = 1
                phi = result["angle"]
                rphi = column_stack((array([radius] * len(phi)), phi))
                Xdatas = rphiz_to_xyz(rphi)

            Ydatas = column_stack(
                (squeeze(result["comp_x"]), squeeze(result["comp_y"]))
            )
        else:
            title = REV_COMP_DICT[component_list[0]].capitalize() + " " + self.name
            if component_list[0] in ["comp_x", "comp_y"]:
                vecfield = self.to_xyz()
                result = vecfield.components[component_list[0]].get_along(
                    arg_list,
                    axis_data=axis_data,
                    unit=unit,
                    is_norm=is_norm,
                )
                if "x" in result and "y" in result:
                    Xdatas = column_stack((result["x"], result["y"]))
                else:
                    if radius is None:
                        radius = 1
                    phi = result["angle"]
                    rphi = column_stack((array([radius] * len(phi)), phi))
                    Xdatas = rphiz_to_xyz(rphi)
                Y_comp = result[vecfield.components[component_list[0]].symbol]
                if component_list[0] == "comp_x":
                    Y_comp[Xdatas[:, 0] < 0] = -Y_comp[Xdatas[:, 0] < 0]
                    Ydatas = column_stack((Y_comp, 0 * Y_comp, 0 * Y_comp))
                else:
                    Y_comp[Xdatas[:, 1] < 0] = -Y_comp[Xdatas[:, 1] < 0]
                    Ydatas = column_stack((0 * Y_comp, Y_comp, 0 * Y_comp))
            else:
                vecfield = self.to_rphiz()
                result = vecfield.components[component_list[0]].get_along(
                    arg_list,
                    axis_data=axis_data,
                    unit=unit,
                    is_norm=is_norm,
                )
                Y_comp = result[vecfield.components[component_list[0]].symbol]

                if "x" in result and "y" in result:
                    Xdatas = column_stack((result["x"], result["y"]))
                    rphiz = xyz_to_rphiz(Xdatas)

                    if component_list[0] == "radial":
                        Y_3d = column_stack((Y_comp, 0 * Y_comp, 0 * Y_comp))
                    else:
                        Y_3d = column_stack((0 * Y_comp, Y_comp, 0 * Y_comp))

                    Ydatas = rphiz_to_xyz_field(Y_3d, rphiz[:, 1])

                else:
                    if radius is None:
                        radius = 1
                    phi = result["angle"]
                    rphi = column_stack((array([radius] * len(phi)), phi))
                    Xdatas = rphiz_to_xyz(rphi)
                    if component_list[0] == "radial":
                        Y_3d = column_stack((Y_comp, 0 * Y_comp, 0 * Y_comp))
                    else:
                        Y_3d = column_stack((0 * Y_comp, Y_comp, 0 * Y_comp))

                    Ydatas = rphiz_to_xyz_field(Y_3d, phi)

        Ydatas = real(Ydatas * exp(1j * phase))

        title2 = " for "
        if "time" in result:
            if isinstance(result["time"], str):
                time_str = result["time"]
            else:
                time_str = array2string(
                    result["time"], formatter={"float_kind": "{:.3g}".format}
                )
            title2 += (
                "time="
                + time_str.replace(" ", ", ").replace("[", "").replace("]", "")
                + "[s]"
            )
        elif "freqs" in result:
            title2 += (
                "freqs="
                + array2string(
                    result["freqs"], formatter={"float_kind": "{:.5g}".format}
                )
                .replace(" ", ", ")
                .replace("[", "")
                .replace("]", "")
                + "[Hz]"
            )

        title3 = ""
        axes_dict_other = result["axes_dict_other"]
        for axis_name in axes_dict_other:
            is_display = True
            for axis in self.get_axes():
                if axis.name == axis_name:
                    if isinstance(axis, DataPattern) and len(axis.unique_indices) == 1:
                        is_display = False
            if is_display:
                if isinstance(axes_dict_other[axis_name][0], str):
                    title3 += axis_name + "=" + axes_dict_other[axis_name][0]
                else:
                    title3 += (
                        axis_name
                        + "="
                        + array2string(
                            axes_dict_other[axis_name][0],
                            formatter={"float_kind": "{:.3g}".format},
                        )
                        .replace(" ", ", ")
                        .replace("[", "")
                        .replace("]", "")
                        + " ["
                        + axes_dict_other[axis_name][1]
                        + "], "
                    )
        if title2 == " for " and title3 == "":
            title2 = ""
        title = title + title2 + title3

        # Normalize Ydatas
        # Ydatas = Ydatas / (np_max(Ydatas) * 0.001 * radius)
        plot_2D(
            [Xdatas],
            [Ydatas],
            color_list=color_list,
            linestyle_list=linestyles,
            linewidth_list=linewidth_list,
            title=title,
            legend_list=[self.name],
            xlabel="[m]",
            ylabel="[m]",
            fig=fig,
            ax=ax,
            type_plot="quiver",
            save_path=save_path,
            is_show_fig=is_show_fig,
            win_title=win_title,
            font_name=font_name,
            font_size_title=font_size_title,
            font_size_label=font_size_label,
            font_size_legend=font_size_legend,
            is_show_legend=is_show_legend,
            is_grid=False,
            scale_units=scale_units,
            scale=scale,
            width=width,
            is_outside_legend=is_outside_legend,
            is_frame_legend=is_frame_legend,
        )

    else:

        # Call the plot on each component
        if component_list is None:  # default: extract all components
            component_list = self.components.keys()
        for i, comp in enumerate(component_list):
            # (fig, axes, patch_leg, label_leg) = init_fig(None, shape="rectangle")

            if save_path is not None and len(component_list) > 1:
                save_path_comp = (
                    save_path.split(".")[0] + "_" + comp + "." + save_path.split(".")[1]
                )
            else:
                save_path_comp = save_path

            self.components[comp].plot_2D_Data(
                arg_list,
                axis_data=axis_data,
                is_norm=is_norm,
                unit=unit,
                data_list=[dat.components[comp] for dat in data_list],
                legend_list=legend_list,
                linestyles=linestyles,
                color_list=color_list,
                save_path=save_path_comp,
                x_min=x_min,
                x_max=x_max,
                y_min=y_min,
                y_max=y_max,
                is_logscale_x=is_logscale_x,
                is_logscale_y=is_logscale_y,
                is_disp_title=is_disp_title,
                is_grid=is_grid,
                is_auto_ticks=is_auto_ticks,
                is_auto_range=is_auto_range,
                fig=fig,
                ax=ax,
                barwidth=barwidth,
                type_plot=type_plot,
                fund_harm_dict=fund_harm_dict,
                is_show_fig=is_show_fig,
                thresh=thresh,
                font_name=font_name,
                font_size_title=font_size_title,
                font_size_label=font_size_label,
                font_size_legend=font_size_legend,
                is_show_legend=is_show_legend,
                is_outside_legend=is_outside_legend,
                is_frame_legend=is_frame_legend,
            )



import numpy as np
import scipy.interpolate as scp_int

from SciDataTool.Functions.set_routines import unique_tol


def change_referential_spectrum(
    freqs,
    wavenumbers_circ,
    rotation_speed,
    spectrum,
    atol=1e-9,
    freqs_new=np.array([]),
    I1=np.array([]),
    Irf_un=np.array([]),
    is_double_f0=False,
    atol_freq=1e-6,
):
    """Compute a new 2D spectrum depending on a rotating referential defined by a speed.

    Parameters
    ----------
    freqs : ndarray
        frequencies vector
    wavenumbers_circ : ndarray
        circumferential wavenumbers vector
    rotation_speed : float
        rotation speed
    spectrum : ndarray
        2D/3D spectrum with freqs/wavenumbers_circ as two first axes
    atol: float
        Absolute tolerance under which amplitudes are assumed to be 0
    freqs_new : ndarray
        frequencies vector in the new rotating referential
    I1 : ndarray
        Array of component indices in new spectrum
    Irf_un: ndarray
        Array of indices of unique frequency/wavenumber couples
    is_double_f0: bool
        True to multiply spectrum components which have zero frequency and non-zero wavenumber
    atol_freq: float
        Absolute tolerance under which frequencies are assumed to be equal

    Returns
    -------
    spectrum_new : ndarray
        spectrum in the new rotating referential
    freqs_new : ndarray
        frequencies vector in the new rotating referential
    I1 : ndarray
        Array of component indices in new spectrum
    Irf_un: ndarray
        Array of indices of unique frequency/wavenumber couples

    """
    Nf = freqs.size
    Nr = wavenumbers_circ.size
    # Get number of slices depending on input spectrum shape
    if spectrum.ndim > 2:
        Nslice = spectrum.shape[2]
        is_squeeze = False
    else:
        Nslice = 1
        is_squeeze = True
        spectrum = spectrum[:, :, None]

    if freqs_new.size == 0:
        # Calculate new frequency values by shifting frequencies
        Xwavenb, Xfreqs = np.meshgrid(wavenumbers_circ, freqs)
        Xfreqs_new = Xfreqs + Xwavenb * rotation_speed / 60

        # Get unique frequencies
        freqs_new, If0 = unique_tol(
            Xfreqs_new.ravel("C"),
            return_inverse=True,
            axis=0,
            tol=atol_freq,
            is_abs_tol=True,
        )

        # Get frequency/wavenumber_circ position in new matrix [Nf_new, Nr]
        Ir0 = np.tile(np.arange(Nr, dtype=int), Nf)
        Irf = np.concatenate((If0[:, None], Ir0[:, None]), axis=1)

        # Get unique couples of frequency/wavenumber to sum on same harmonics
        Irf_un, I1 = np.unique(Irf, return_inverse=True, axis=0)

    # Number of frequencies in new referential
    Nf_new = freqs_new.size

    if is_double_f0:
        # Multiply by two spectrum components which have f=0, r!=0
        jf0 = np.abs(freqs) < 1e-4
        jr = wavenumbers_circ != 0
        spectrum[jf0, jr, :] = 2 * spectrum[jf0, jr, :]

    # Calculate spectrum amplitude in new referential by summing all contributions
    # which have the same orders and wavenumber for each slice
    spectrum_new = np.zeros((Nf_new, Nr, Nslice), dtype=spectrum.dtype)
    for k in range(Nslice):
        # Reshape values for kth slice columnwise
        amp_k = spectrum[:, :, k].ravel("C")
        # Sum all contributions which have the same orders and wavenumber as given by I1
        if spectrum.dtype == complex:
            # bincount fails on complex numbers, real and imaginary parts must be treated separately
            amp_new_k = np.bincount(I1, weights=amp_k.real) + 1j * np.bincount(
                I1, weights=amp_k.imag
            )
        else:
            amp_new_k = np.bincount(I1, weights=amp_k)
        # Store amplitudes at new frequency/wavenumber positions
        spectrum_new[Irf_un[:, 0], Irf_un[:, 1], k] = amp_new_k

    if is_double_f0:
        # Divide by two spectrum components which have f=0, r!=0
        spectrum[jf0, jr, :] = spectrum[jf0, jr, :] / 2

    if atol > 0:
        # Filter harmonics that are below input absolute tolerance
        Imask = (
            np.sum(np.sum(np.abs(spectrum_new), axis=2), axis=1)
            > np.max(np.abs(spectrum_new)) * atol
        )
        spectrum_new = spectrum_new[Imask, ...]
        freqs_new = freqs_new[Imask]

    if is_squeeze:
        # Squeeze spectrum back to 2D
        spectrum_new = spectrum_new[:, :, 0]

    return spectrum_new, freqs_new, I1, Irf_un


def change_referential_waveform(
    val0,
    time0,
    angle0,
    rotation_speed,
    is_aper_a=False,
    is_aper_t=False,
    ta_in=tuple(),
    ta_out=tuple(),
):
    """Change referential of input 3D array defined on time, angle and z given input rotation speed
    (algebric to include rotation direction)

    Parameters
    ----------
    val0 : ndarray
        Field values in new referential
    time0 : ndarray
        time vector [s]
    angle0 : float
        angle vector [rad]
    rotation_speed : float
        rotation speed [rpm]
    per_a: int
        angle periodicity number (one period)
    is_aper_a: bool
        True if there is a spatial anti-periodicity
    is_aper_t: bool
        True if there is a time anti-periodicity
    ta_in: tuple
        Tuple of input time/angle meshgrids
    ta_out: tuple
        Tuple of output time/angle meshgrids

    Returns
    -------
    val_new : ndarray
        Field values in new referential
    time_new : ndarray
        time vector in new referential [s]
    angle_new : ndarray
        angle vector in new referential [rad]
    ta_in: tuple
        Tuple of input time/angle meshgrids
    ta_out: tuple
        Tuple of output time/angle meshgrids
    """

    # Init size
    Nt = time0.size
    Na = angle0.size
    if val0.ndim > 2:
        Nslice = val0.shape[2]
        is_squeeze = False
    else:
        Nslice = 1
        val0 = val0[:, :, None]
        is_squeeze = True
    shape0 = [Nt, Na, Nslice]

    if len(ta_in) == 0 or len(ta_out) == 0:
        # Add final value to time and space vectors
        tf = time0[-1] + time0[1] - time0[0]
        time1 = np.append(time0, tf)
        alphaf = angle0[-1] + angle0[1] - angle0[0]
        angle1 = np.append(angle0, alphaf)
        ta_in = (time1, angle1)

        # Build 2D meshgrids and flatten them columnwise
        Xangle0, Xtime0 = np.meshgrid(angle0, time0)
        Xtime0, Xangle0 = Xtime0.ravel("C"), Xangle0.ravel("C")

        # Shift angle according to rotation speed
        Xangle_new = (Xangle0 + 2 * np.pi * rotation_speed / 60 * Xtime0) % alphaf
        ta_out = (Xtime0, Xangle_new)

    # 2D interpolate for new angle array
    val_new = np.zeros(shape0)
    valk = np.zeros((Nt + 1, Na + 1))
    for k in range(Nslice):
        # Make current slice periodic along time and space
        valk[0:Nt, 0:Na] = val0[:, :, k]
        valk[-1, :-1] = val0[0, :, k]
        valk[:-1, -1] = val0[:, 0, k]
        valk[-1, -1] = val0[0, 0, k]

        # Perform 2D interpolation
        val_new[:, :, k] = scp_int.RegularGridInterpolator(
            ta_in, valk, method="linear"
        )(ta_out).reshape((Nt, Na))

    if is_aper_t:
        # Remove half part of 1st dimension
        val_new = val_new[: int(Nt / 2), :, :]
        time_new = time0[: int(Nt / 2)]
    else:
        time_new = time0

    if is_aper_a:
        # Remove half part of 2nd dimension
        val_new = val_new[:, : int(Na / 2), :]
        angle_new = angle0[: int(Na / 2)]
    else:
        angle_new = angle0

    if is_squeeze:
        # Remove 3rd dimension
        val_new = val_new[:, :, 0]

    return val_new, time_new, angle_new, ta_in, ta_out



# -*- coding: utf-8 -*-

# File generated according to WAxisManager.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from SciDataTool.GUI.WAxisSelector.WAxisSelector import WAxisSelector


class Ui_WAxisManager(object):
    def setupUi(self, WAxisManager):
        if not WAxisManager.objectName():
            WAxisManager.setObjectName("WAxisManager")
        WAxisManager.resize(461, 266)
        self.verticalLayout_2 = QVBoxLayout(WAxisManager)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.g_axes = QGroupBox(WAxisManager)
        self.g_axes.setObjectName("g_axes")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.g_axes.sizePolicy().hasHeightForWidth())
        self.g_axes.setSizePolicy(sizePolicy)
        self.g_axes.setMinimumSize(QSize(0, 0))
        self.g_axes.setMaximumSize(QSize(16777215, 16777215))
        self.lay_axes = QHBoxLayout(self.g_axes)
        self.lay_axes.setSpacing(4)
        self.lay_axes.setObjectName("lay_axes")
        self.lay_axes.setContentsMargins(5, 0, 5, 0)
        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QLabel(self.g_axes)
        self.label.setObjectName("label")
        self.label.setMinimumSize(QSize(0, 24))

        self.verticalLayout.addWidget(self.label)

        self.label_4 = QLabel(self.g_axes)
        self.label_4.setObjectName("label_4")
        self.label_4.setMinimumSize(QSize(0, 18))

        self.verticalLayout.addWidget(self.label_4)

        self.label_2 = QLabel(self.g_axes)
        self.label_2.setObjectName("label_2")
        self.label_2.setMinimumSize(QSize(0, 20))

        self.verticalLayout.addWidget(self.label_2)

        self.label_3 = QLabel(self.g_axes)
        self.label_3.setObjectName("label_3")
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setMinimumSize(QSize(0, 18))

        self.verticalLayout.addWidget(self.label_3)

        self.label_5 = QLabel(self.g_axes)
        self.label_5.setObjectName("label_5")
        self.label_5.setMinimumSize(QSize(0, 23))

        self.verticalLayout.addWidget(self.label_5)

        self.lay_axes.addLayout(self.verticalLayout)

        self.w_axis_1 = WAxisSelector(self.g_axes)
        self.w_axis_1.setObjectName("w_axis_1")
        sizePolicy.setHeightForWidth(self.w_axis_1.sizePolicy().hasHeightForWidth())
        self.w_axis_1.setSizePolicy(sizePolicy)
        self.w_axis_1.setMinimumSize(QSize(150, 0))
        self.w_axis_1.setMaximumSize(QSize(150, 16777215))

        self.lay_axes.addWidget(self.w_axis_1)

        self.w_axis_2 = WAxisSelector(self.g_axes)
        self.w_axis_2.setObjectName("w_axis_2")
        sizePolicy.setHeightForWidth(self.w_axis_2.sizePolicy().hasHeightForWidth())
        self.w_axis_2.setSizePolicy(sizePolicy)
        self.w_axis_2.setMinimumSize(QSize(150, 0))
        self.w_axis_2.setMaximumSize(QSize(150, 16777215))

        self.lay_axes.addWidget(self.w_axis_2)

        self.verticalLayout_2.addWidget(self.g_axes)

        self.g_data_extract = QGroupBox(WAxisManager)
        self.g_data_extract.setObjectName("g_data_extract")
        sizePolicy.setHeightForWidth(
            self.g_data_extract.sizePolicy().hasHeightForWidth()
        )
        self.g_data_extract.setSizePolicy(sizePolicy)
        self.g_data_extract.setMinimumSize(QSize(0, 0))
        self.g_data_extract.setMaximumSize(QSize(16777215, 16777215))
        self.lay_data_extract = QVBoxLayout(self.g_data_extract)
        self.lay_data_extract.setObjectName("lay_data_extract")

        self.verticalLayout_2.addWidget(self.g_data_extract)

        self.retranslateUi(WAxisManager)

        QMetaObject.connectSlotsByName(WAxisManager)

    # setupUi

    def retranslateUi(self, WAxisManager):
        WAxisManager.setWindowTitle(
            QCoreApplication.translate("WAxisManager", "WAxisManager", None)
        )
        self.g_axes.setTitle(
            QCoreApplication.translate("WAxisManager", "Axes Selection", None)
        )
        self.label.setText("")
        self.label_4.setText(QCoreApplication.translate("WAxisManager", "Axis", None))
        self.label_2.setText(QCoreApplication.translate("WAxisManager", "Action", None))
        self.label_3.setText(QCoreApplication.translate("WAxisManager", "Unit", None))
        self.label_5.setText("")
        self.g_data_extract.setTitle(
            QCoreApplication.translate("WAxisManager", "Axes Operations", None)
        )

    # retranslateUi



def to_rphiz(self):
    """Performs the corrdinate change and stores the resulting field in a VectorField object.
    Parameters
    ----------
    self : VectorField
        a VectorField object
    Returns
    -------
    a VectorField object
    """

    # Dynamic import to avoid loop
    module = __import__("SciDataTool.Classes.VectorField", fromlist=["VectorField"])
    VectorField = getattr(module, "VectorField")

    if "radial" in self.components or "tangential" in self.components:
        return self.copy()

    else:
        # Coordinate transform
        arg_list = [
            axis.name
            if axis.name in ["freqs", "wavenumber"]
            else axis.name + "[smallestperiod]"
            for axis in self.components["comp_x"].axes
        ]
        result = self.get_rphiz_along(*arg_list)
        # Store in new VectorField
        comp_dict = dict()

        Comp_r = self.components["comp_x"].copy()
        Comp_r.name = "Radial " + self.components["comp_x"].name.lower().replace(
            " along x-axis", ""
        )
        Comp_r.symbol = self.components["comp_x"].symbol.replace("_x", "_r")
        Comp_r.values = result["radial"]
        comp_dict["radial"] = Comp_r

        Comp_t = self.components["comp_x"].copy()
        Comp_t.name = "Tangential " + self.components["comp_x"].name.lower().replace(
            " along x-axis", ""
        )
        Comp_t.symbol = self.components["comp_x"].symbol.replace("_r", "_y")
        Comp_t.values = result["tangential"]
        comp_dict["comp_y"] = Comp_t

        if "axial" in self.components:
            Comp_z = self.components["axial"].copy()
            comp_dict["comp_z"] = Comp_z

        return VectorField(name=self.name, symbol=self.symbol, components=comp_dict)



import numpy as np


def union1d_tol(ar1, ar2, tol=1e-6, is_abs_tol=False, return_indices=False):
    """Return the intersect1d values of the input array a given tolerance tol

    Parameters
    ----------
    ar1 : ndarray
        First array to union values
    ar2 : ndarray
        Second array to union values
    tol : Float
        the tolerance value
    is_abs_tol : bool
        True to apply absolute tolerance regarding maximum absolute value in (ar1, ar2)
    return_indices : bool
        True to return indices

    Returns
    -------
    b : ndarray
        array containing unioned values
    Ia : ndarray
        index array such as ar1 = b[Ia]
    Ib : ndarray
        index array such as ar2 = b[Ib]
    """

    # Get union values as unique values in both arrays
    b = unique_tol(
        np.concatenate((ar1, ar2)),
        tol=tol,
        is_abs_tol=is_abs_tol,
        return_index=False,
        return_inverse=False,
    )

    if return_indices:
        # Map union values with values in 1st array
        Ia = np.argmin(np.abs(b[:, None] - ar1[None, :]), axis=0)
        # Map union values with values in 2nd array
        Ib = np.argmin(np.abs(b[:, None] - ar2[None, :]), axis=0)

        return b, Ia, Ib

    else:
        return b


def unique_tol(
    a,
    tol=1e-6,
    is_abs_tol=False,
    return_index=False,
    return_inverse=False,
    return_counts=False,
    is_stable=False,
    axis=0,
):
    """Return the unique values of the input array a given tolerance tol

    Parameters
    ----------
    a : ndarray
        the array to calculate unique values
    tol : Float
        the tolerance value
    is_abs_tol : bool
        True to apply absolute tolerance regarding maximum absolute value in a
    return_index : bool
        to return direct index
    return_inverse : bool
        to return inverse index
    return_counts : bool
        to return occurences count
    is_stable: bool
        to return unique array with stable order (not sorted)
    axis : int
        Axis index on which to calculate unique values

    Returns
    -------
    b : ndarray
        array containing unique values
    Ia : ndarray
        direct index array such as b=a[Ia] if is stable, else b=sort(a[Ia])
    Ib: ndarray
        inverse index array such as a=b[Ib]
    count0: ndarray
        array counting occurences for each unique value
    """

    if not is_abs_tol:
        tol = get_relative_tolerance(a, tol)

    res_tuple = np.unique(
        np.round(a / tol),
        return_index=True,
        return_inverse=return_inverse,
        return_counts=return_counts,
        axis=axis,
    )

    if is_stable:
        Ia = np.sort(res_tuple[1])
    else:
        Ia = res_tuple[1]

    b = a[Ia]

    if return_inverse:
        Ib = res_tuple[2]

    if return_counts:
        count0 = res_tuple[3]

    if return_index and return_inverse and return_counts:
        return b, Ia, Ib, count0
    elif return_index and return_inverse:
        return b, Ia, Ib
    elif return_index and return_counts:
        return b, Ia, count0
    elif return_inverse and return_counts:
        return b, Ib, count0
    elif return_index:
        return b, Ia
    elif return_inverse:
        return b, Ib
    elif return_counts:
        return b, count0
    else:
        return b


def intersect1d_tol(
    ar1, ar2, tol=1e-6, is_abs_tol=False, return_indices=False, assume_unique=False
):
    """Return the intersect1d values of the input array a given tolerance tol

    Parameters
    ----------
    ar1 : ndarray
        First array to intersect values
    ar2 : ndarray
        Second array to intersect values
    tol : Float
        the tolerance value
    is_abs_tol : bool
        True to consider absolute tolerance, False to consider relative tolerance
    return_indices : bool
        True to return original indices for both input arrays
    assume_unique : bool
        True to assume values are unique in both input arrays

    Returns
    -------
    b : ndarray
        array containing common values
    Ia : ndarray
        array of indices such as b = ar1[Ia]
    Ib : ndarray
        array of indices such as b = ar2[Ib]
    """

    if not is_abs_tol:
        tol = get_relative_tolerance(ar1, tol)

    _, Ia, Ib = np.intersect1d(
        np.floor(ar1 / tol),
        np.floor(ar2 / tol),
        assume_unique=assume_unique,
        return_indices=True,
    )

    b = ar1[Ia]

    if return_indices:
        return b, Ia, Ib
    else:
        return b


def get_relative_tolerance(a, atol):
    """Calculate relative tolerance given an array a and an absolute tolerance

    Parameters
    ----------
    a : ndarray
        array for which to calculate relative tolerance
    atol : float
        absolute tolerance

    Returns
    -------
    rtol : float
        relative tolerance
    """

    a_max = np.max(np.abs(a))
    if a_max >= 0:
        rtol = atol * np.max(np.abs(a))

    if rtol > 1 or rtol == 0:
        # threshold tol to 1
        rtol = 1

    return rtol



# -*- coding: utf-8 -*-

# File generated according to WDataRange.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from SciDataTool.GUI.Tools.FloatEdit import FloatEdit


class Ui_WDataRange(object):
    def setupUi(self, WDataRange):
        if not WDataRange.objectName():
            WDataRange.setObjectName("WDataRange")
        WDataRange.resize(300, 132)
        self.verticalLayout = QVBoxLayout(WDataRange)
        self.verticalLayout.setObjectName("verticalLayout")
        self.g_range = QGroupBox(WDataRange)
        self.g_range.setObjectName("g_range")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.g_range.sizePolicy().hasHeightForWidth())
        self.g_range.setSizePolicy(sizePolicy)
        self.g_range.setMinimumSize(QSize(0, 0))
        self.g_range.setMaximumSize(QSize(16777215, 16777215))
        self.gridLayout = QGridLayout(self.g_range)
        self.gridLayout.setObjectName("gridLayout")
        self.in_unit = QLabel(self.g_range)
        self.in_unit.setObjectName("in_unit")

        self.gridLayout.addWidget(self.in_unit, 0, 0, 1, 1)

        self.c_unit = QComboBox(self.g_range)
        self.c_unit.setObjectName("c_unit")
        self.c_unit.setMinimumSize(QSize(0, 20))

        self.gridLayout.addWidget(self.c_unit, 0, 1, 1, 1)

        self.in_min = QLabel(self.g_range)
        self.in_min.setObjectName("in_min")

        self.gridLayout.addWidget(self.in_min, 1, 0, 1, 1)

        self.lf_min = FloatEdit(self.g_range)
        self.lf_min.setObjectName("lf_min")
        sizePolicy1 = QSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
        sizePolicy1.setHorizontalStretch(70)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.lf_min.sizePolicy().hasHeightForWidth())
        self.lf_min.setSizePolicy(sizePolicy1)
        self.lf_min.setMinimumSize(QSize(0, 20))
        self.lf_min.setMaximumSize(QSize(70, 16777215))

        self.gridLayout.addWidget(self.lf_min, 1, 1, 1, 1)

        self.in_max = QLabel(self.g_range)
        self.in_max.setObjectName("in_max")

        self.gridLayout.addWidget(self.in_max, 2, 0, 1, 1)

        self.lf_max = FloatEdit(self.g_range)
        self.lf_max.setObjectName("lf_max")
        sizePolicy1.setHeightForWidth(self.lf_max.sizePolicy().hasHeightForWidth())
        self.lf_max.setSizePolicy(sizePolicy1)
        self.lf_max.setMinimumSize(QSize(0, 20))
        self.lf_max.setMaximumSize(QSize(70, 16777215))

        self.gridLayout.addWidget(self.lf_max, 2, 1, 1, 1)

        self.verticalLayout.addWidget(self.g_range)

        self.retranslateUi(WDataRange)

        QMetaObject.connectSlotsByName(WDataRange)

    # setupUi

    def retranslateUi(self, WDataRange):
        WDataRange.setWindowTitle("")
        self.g_range.setTitle(QCoreApplication.translate("WDataRange", "Range", None))
        self.in_unit.setText(QCoreApplication.translate("WDataRange", "unit", None))
        self.in_min.setText(QCoreApplication.translate("WDataRange", "min", None))
        self.lf_min.setText(QCoreApplication.translate("WDataRange", "0.314", None))
        self.in_max.setText(QCoreApplication.translate("WDataRange", "max", None))
        self.lf_max.setText(QCoreApplication.translate("WDataRange", "0.314", None))

    # retranslateUi



from PySide2.QtWidgets import QWidget
import matplotlib.pyplot as plt

from SciDataTool.GUI.WDataRange.Ui_WDataRange import Ui_WDataRange
from PySide2.QtCore import Signal
from SciDataTool.Functions import parser
from SciDataTool.GUI import update_cb_enable


class WDataRange(Ui_WDataRange, QWidget):
    """Widget to select the Data/output range"""

    refreshNeeded = Signal()

    def __init__(self, parent=None):
        """Linking the button with their method + initializing the arguments used

        Parameters
        ----------
        self : WDataRange
            a WDataRange object
        parent : QWidget
            The parent QWidget
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self, parent=parent)
        self.setupUi(self)
        self.name = ""
        self.unit_list = list()
        self.unit = None

        self.c_unit.currentTextChanged.connect(self.update_unit)
        self.lf_min.editingFinished.connect(self.update_needed)
        self.lf_max.editingFinished.connect(self.update_needed)

    def get_field_selected(self):
        """Method that will sent the parameters on the field selected by the user (unit and min/max)
        Parameters
        ----------
        self : WDataRange
            a WDataRange object
        Output
        ---------
        string
            name of the action on the field
        """

        return {
            "unit": self.c_unit.currentText(),
            "min": self.lf_min.value(),
            "max": self.lf_max.value(),
        }

    def set_min_max(self):
        """Method that will set the FloatEdit of the widget that are responsible for the min value and for the max value.
        Parameters
        ----------
        self : WDataRange
            a WDataRange object
        """

        field_min = None
        field_max = None

        # Get limits from figure
        fig = plt.gcf()
        if fig is not None:
            if len(fig.axes) == 1:
                field_min = fig.axes[0].get_ylim()[0]
                field_max = fig.axes[0].get_ylim()[1]
            else:
                field_min = fig.axes[1].dataLim.extents[0]
                field_max = fig.axes[1].dataLim.extents[-1]

        if field_min is not None:
            is_dB = "dB" in self.unit
            self.lf_min.setValue(field_min, is_dB)
            self.lf_max.setValue(field_max, is_dB)
        else:
            self.lf_min.clear()
            self.lf_max.clear()

    def set_name(self, field_name):
        """Method that set the name of the widget which is the name of the field that we are plotting

        Parameters
        ----------
        self : WDataRange
            a WDataRange object
        field : DataND
            the data object that hold the field that we want to plot
        """
        self.name = field_name

    def set_range(self, data, unit=None):
        """Method that set the data range widget with the value from the DataND object

        Parameters
        ----------
        self : WDataRange
            a WDataRange object
        data : DataND
            the data object that hold the field that will set the widget
        """
        if unit is not None:
            self.unit = unit
            self.set_unit(data)
            if unit != "SI":  # Adding unit to unit combobox
                if self.c_unit.currentText() != unit:
                    self.c_unit.insertItem(0, unit)
            if unit == "dBA" and self.c_unit.count() > 1:  # Also adding dB
                self.c_unit.insertItem(1, "dB")
            self.c_unit.setCurrentIndex(0)
            update_cb_enable(self.c_unit)
        # else:
        #     self.set_unit(data)
        self.lf_min.clear()
        self.lf_max.clear()

    def set_range_user_input(
        self,
        z_min=None,
        z_max=None,
    ):
        """Method that modify the unit selected and the floatEdit according to the inputs given by the user (auto-plot)
        Parameters
        ----------
        self : WDataRange
            a WDataRange object
        z_min : float
            Minimum value for Z axis (or Y if only one axe)
        z_max : float
            Minimum value for Z axis (or Y if only one axe)
        """
        is_dB = "dB" in self.unit
        if z_max is not None:
            # Setting max float edit
            self.lf_max.setValue(z_max, is_dB)
        if z_min is not None:
            # Setting min float edit
            self.lf_min.setValue(z_min, is_dB)

    def set_unit(self, field):
        """Method that set the unit combobox according to the unit of the field that we are plotting

        Parameters
        ----------
        self : WDataRange
            a WDataRange object
        data : DataND
            the data object that hold the field that we want to plot
        """

        # # Updating the unit combobox
        self.c_unit.clear()
        self.c_unit.addItem(field.unit)
        update_cb_enable(self.c_unit)
        self.unit = field.unit

    def update_unit(self):
        """Method that clears min/max then calls update_needed
        Parameters
        ----------
        self : WDataRange
            a WDataRange object
        """
        self.lf_max.clear()
        self.lf_min.clear()
        self.update_needed()

    def update_needed(self):
        """Method that emit a signal to automatically refresh the plot inside the GUI
        Parameters
        ----------
        self : WDataRange
            a WDataRange object
        """
        # Making sure that we always have min < max
        if self.lf_min.value() != None and self.lf_max.value() != None:
            if self.lf_min.value() > self.lf_max.value():
                is_dB = "dB" in self.unit
                self.blockSignals(True)
                temp = self.lf_max.value()
                self.lf_max.setValue(self.lf_min.value(), is_dB)
                self.lf_min.setValue(temp, is_dB)
                self.blockSignals(False)

        self.refreshNeeded.emit()



# -*- coding: utf-8 -*-

# File generated according to WPlotManager.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from SciDataTool.GUI.WDataRange.WDataRange import WDataRange
from SciDataTool.GUI.WAxisManager.WAxisManager import WAxisManager
from SciDataTool.GUI.WVectorSelector.WVectorSelector import WVectorSelector


class Ui_WPlotManager(object):
    def setupUi(self, WPlotManager):
        if not WPlotManager.objectName():
            WPlotManager.setObjectName("WPlotManager")
        WPlotManager.resize(376, 683)
        self.verticalLayout = QVBoxLayout(WPlotManager)
        self.verticalLayout.setObjectName("verticalLayout")
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.w_vect_selector = WVectorSelector(WPlotManager)
        self.w_vect_selector.setObjectName("w_vect_selector")
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.w_vect_selector.sizePolicy().hasHeightForWidth()
        )
        self.w_vect_selector.setSizePolicy(sizePolicy)
        self.w_vect_selector.setMinimumSize(QSize(0, 0))
        self.w_vect_selector.setMaximumSize(QSize(16777215, 16777215))

        self.verticalLayout.addWidget(self.w_vect_selector)

        self.w_axis_manager = WAxisManager(WPlotManager)
        self.w_axis_manager.setObjectName("w_axis_manager")
        sizePolicy.setHeightForWidth(
            self.w_axis_manager.sizePolicy().hasHeightForWidth()
        )
        self.w_axis_manager.setSizePolicy(sizePolicy)
        self.w_axis_manager.setMinimumSize(QSize(0, 0))
        self.w_axis_manager.setMaximumSize(QSize(16777215, 16777215))

        self.verticalLayout.addWidget(self.w_axis_manager)

        self.w_range = WDataRange(WPlotManager)
        self.w_range.setObjectName("w_range")
        sizePolicy.setHeightForWidth(self.w_range.sizePolicy().hasHeightForWidth())
        self.w_range.setSizePolicy(sizePolicy)
        self.w_range.setMinimumSize(QSize(0, 0))
        self.w_range.setMaximumSize(QSize(16777215, 16777215))

        self.verticalLayout.addWidget(self.w_range)

        self.gridLayout_2 = QGridLayout()
        self.gridLayout_2.setSpacing(5)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.gridLayout_2.setContentsMargins(11, 11, 10, 11)
        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.gridLayout_2.addItem(self.horizontalSpacer, 0, 0, 1, 1)

        self.b_export = QPushButton(WPlotManager)
        self.b_export.setObjectName("b_export")
        sizePolicy.setHeightForWidth(self.b_export.sizePolicy().hasHeightForWidth())
        self.b_export.setSizePolicy(sizePolicy)
        self.b_export.setMinimumSize(QSize(0, 0))
        self.b_export.setMaximumSize(QSize(16777215, 16777215))

        self.gridLayout_2.addWidget(self.b_export, 0, 1, 1, 1)

        self.verticalLayout.addLayout(self.gridLayout_2)

        self.verticalSpacer = QSpacerItem(
            20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout.addItem(self.verticalSpacer)

        self.retranslateUi(WPlotManager)

        QMetaObject.connectSlotsByName(WPlotManager)

    # setupUi

    def retranslateUi(self, WPlotManager):
        WPlotManager.setWindowTitle(
            QCoreApplication.translate("WPlotManager", "WPlotManager", None)
        )
        self.b_export.setText(
            QCoreApplication.translate("WPlotManager", "Export", None)
        )

    # retranslateUi



import numpy as np

from SciDataTool.Functions.Load.import_class import import_class


def change_periodicity(
    self, per_a_new, per_t_new, is_aper_a_new=None, is_aper_t_new=False
):
    """Change (anti-)periodicities of vectorfield given input periodicities before changing referential

    Parameters
    ----------
    self : VectorField
        A VectorField object
    per_a_new : int
        New number of space periodicities of the machine over 2*pi
    per_t_new : int
        New number of time periodicities of the machine over time period (p/felec by default if Nrev is None) in static or rotating referential
    is_aper_a_new : bool
        True if spatial anti-periodicity in requested for new vectorfield
    is_aper_t_new : bool
        True if time anti-periodicity in requested for new vectorfield

    Returns
    -------
    vf_new : VectorField
        Vectorfield converted with new periodicities
    """

    DataLinspace = import_class("SciDataTool.Classes", "DataLinspace")

    # Get axes list in original VectorField
    # Assumes axes are the following: time, angle, z and z is DataPattern
    axes_list = self.get_axes()

    # Get replicating number for time axis
    per_t, is_aper_t = axes_list[0].get_periodicity()
    per_t = int(per_t / 2) if is_aper_t else per_t
    Nrep_t = int(per_t / per_t_new)
    Nt = Nrep_t * axes_list[0].get_length(
        is_oneperiod=Nrep_t > 1 or (Nrep_t == 1 and not is_aper_t_new),
        is_antiperiod=Nrep_t == 1 and is_aper_t_new,
    )

    # Get replicating number for angle axis
    per_a, is_aper_a = axes_list[1].get_periodicity()
    if is_aper_a_new is None:
        is_aper_a_new = is_aper_a
    per_a = int(per_a / 2) if is_aper_a else per_a
    Nrep_a = int(per_a / per_a_new)
    Na = Nrep_a * axes_list[1].get_length(
        is_oneperiod=Nrep_a > 1 or (Nrep_a == 1 and not is_aper_a_new),
        is_antiperiod=Nrep_a == 1 and is_aper_a_new,
    )

    if Nrep_t == 1 and Nrep_a == 1:
        # Return VectorField as it is
        return self.copy()

    # Init periodicity changed VectorField
    vf_new = type(self)(name=self.name, symbol=self.symbol, components=dict())

    # Keep time anti-periodicity if it exists
    if Nrep_t == 1 and is_aper_t_new:
        time_arg = "time[smallestperiod]"
    else:
        time_arg = "time[oneperiod]"

    # Keep spatial anti-periodicity if it exists
    if Nrep_a == 1 and is_aper_a_new:
        angle_arg = "angle[smallestperiod]"
    else:
        angle_arg = "angle[oneperiod]"

    # Get argument list for call to get_rphiz_along on one time and space period
    arg_list = [time_arg, angle_arg, "z[smallestpattern]"]

    # Get axes values on requested axes
    result = self.get_rphiz_along(*arg_list, is_squeeze=False)
    axes_list_new = list()
    for axis in axes_list:
        axis_new = axis.copy()
        # Change time periodicity
        if Nrep_t > 1 and axis.name == "time":
            axis_new.symmetries = {"period": per_t_new}
            time_full = result["time"]
            if isinstance(axis_new, DataLinspace):
                if axis.include_endpoint:
                    t_final = time_full[-1] + time_full[1] - time_full[0]
                else:
                    t_final = axis.final
                axis_new.number = Nt
                axis_new.final = Nrep_t * t_final
                axis_new.include_endpoint = False
            else:
                t_final = time_full[-1] + time_full[1] - time_full[0]
                for k in range(1, Nrep_t):
                    time_full = np.append(time_full, result["time"] + k * t_final)
                axis_new.values = time_full
        # Change angle periodicity
        if Nrep_a > 1 and axis.name == "angle":
            axis_new.symmetries = {"period": per_a_new}
            angle_full = result["angle"]
            if isinstance(axis_new, DataLinspace):
                if axis.include_endpoint:
                    a_final = angle_full[-1] + angle_full[1] - angle_full[0]
                else:
                    a_final = axis.final
                axis_new.number = Na
                axis_new.final = Nrep_a * a_final
                axis_new.include_endpoint = False
            else:
                a_final = angle_full[-1] + angle_full[1] - angle_full[0]
                for k in range(1, Nrep_a):
                    angle_full = np.append(angle_full, result["angle"] + k * a_final)
                axis_new.values = angle_full

        axes_list_new.append(axis_new)

    # Replicate values by the number of repetitions
    for comp, data in self.components.items():

        # Duplicate values
        if Nrep_t > 1 and Nrep_a > 1:
            val_new = np.tile(result[comp], (Nrep_t, Nrep_a, 1))
        elif Nrep_t > 1:
            val_new = np.tile(result[comp], (Nrep_t, 1, 1))
        elif Nrep_a > 1:
            val_new = np.tile(result[comp], (1, Nrep_a, 1))

        # Create new data object with updated periodicities
        data_new = type(data)(
            name=data.name,
            symbol=data.symbol,
            unit=data.unit,
            axes=axes_list_new,
            values=val_new,
            normalizations=data.normalizations.copy(),
            is_real=data.is_real,
        )

        # Store reduced periodicity data in VectorField
        vf_new.components[comp] = data_new

    return vf_new



from SciDataTool.Functions import AxisError


def get_filter(self, filter_dict):
    """Get filtering indices
    Parameters
    ----------
    self: Data1D
        a Data1D object
    filter_dict: dict
        a dict of the values to filter (keep values defined in filter_dict)
    Returns
    -------
    indices: list
        list of indices to use to get corresponding filter
    """

    # Check that keys in filter_dict are correctly defined in self.filter
    if len(filter_dict.keys()) != len(self.filter.keys()):
        raise AxisError("Filter key not defined in Data1D.filter")
    for key in filter_dict:
        if key not in self.filter:
            raise AxisError("Filter key not defined in Data1D.filter")

    # Prepare filtering keys in correct order
    keys = self.check_filter()

    indices = []
    for i, value in enumerate(self.values):
        items = value.split(self.delimiter)
        nb_match = 0
        for j, item in enumerate(items):
            if item in filter_dict[keys[j]]:
                nb_match += 1
        if nb_match == len(keys):
            indices.append(i)

    return indices



# -*- coding: utf-8 -*-

# File generated according to DDataPlotter.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from SciDataTool.GUI.WPlotManager.WPlotManager import WPlotManager


class Ui_DDataPlotter(object):
    def setupUi(self, DDataPlotter):
        if not DDataPlotter.objectName():
            DDataPlotter.setObjectName("DDataPlotter")
        DDataPlotter.setEnabled(True)
        DDataPlotter.resize(1246, 884)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(DDataPlotter.sizePolicy().hasHeightForWidth())
        DDataPlotter.setSizePolicy(sizePolicy)
        DDataPlotter.setCursor(QCursor(Qt.ArrowCursor))
        self.gridLayout = QGridLayout(DDataPlotter)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.is_auto_refresh = QCheckBox(DDataPlotter)
        self.is_auto_refresh.setObjectName("is_auto_refresh")
        sizePolicy1 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(
            self.is_auto_refresh.sizePolicy().hasHeightForWidth()
        )
        self.is_auto_refresh.setSizePolicy(sizePolicy1)
        self.is_auto_refresh.setMinimumSize(QSize(0, 24))
        self.is_auto_refresh.setMaximumSize(QSize(16777215, 24))
        self.is_auto_refresh.setChecked(False)

        self.horizontalLayout_2.addWidget(self.is_auto_refresh)

        self.b_refresh = QPushButton(DDataPlotter)
        self.b_refresh.setObjectName("b_refresh")
        self.b_refresh.setEnabled(True)
        sizePolicy1.setHeightForWidth(self.b_refresh.sizePolicy().hasHeightForWidth())
        self.b_refresh.setSizePolicy(sizePolicy1)
        self.b_refresh.setMinimumSize(QSize(0, 0))
        self.b_refresh.setMaximumSize(QSize(16777215, 16777215))
        self.b_refresh.setLayoutDirection(Qt.LeftToRight)

        self.horizontalLayout_2.addWidget(self.b_refresh)

        self.gridLayout.addLayout(self.horizontalLayout_2, 1, 1, 1, 1)

        self.w_scroll = QScrollArea(DDataPlotter)
        self.w_scroll.setObjectName("w_scroll")
        sizePolicy2 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.w_scroll.sizePolicy().hasHeightForWidth())
        self.w_scroll.setSizePolicy(sizePolicy2)
        self.w_scroll.setMinimumSize(QSize(200, 0))
        self.w_scroll.setMaximumSize(QSize(400, 16777215))
        self.w_scroll.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
        self.w_scroll.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 398, 823))
        self.lay_scroll = QVBoxLayout(self.scrollAreaWidgetContents)
        self.lay_scroll.setObjectName("lay_scroll")
        self.lay_scroll.setContentsMargins(0, 0, 0, 0)
        self.w_plot_manager = WPlotManager(self.scrollAreaWidgetContents)
        self.w_plot_manager.setObjectName("w_plot_manager")
        sizePolicy1.setHeightForWidth(
            self.w_plot_manager.sizePolicy().hasHeightForWidth()
        )
        self.w_plot_manager.setSizePolicy(sizePolicy1)
        self.w_plot_manager.setMinimumSize(QSize(0, 0))

        self.lay_scroll.addWidget(self.w_plot_manager)

        self.w_scroll.setWidget(self.scrollAreaWidgetContents)

        self.gridLayout.addWidget(self.w_scroll, 0, 1, 1, 1)

        self.plot_layout = QVBoxLayout()
        self.plot_layout.setObjectName("plot_layout")

        self.gridLayout.addLayout(self.plot_layout, 0, 0, 2, 1)

        self.retranslateUi(DDataPlotter)

        QMetaObject.connectSlotsByName(DDataPlotter)

    # setupUi

    def retranslateUi(self, DDataPlotter):
        DDataPlotter.setWindowTitle(
            QCoreApplication.translate("DDataPlotter", "Data Plot", None)
        )
        self.is_auto_refresh.setText(
            QCoreApplication.translate("DDataPlotter", "Auto Refresh", None)
        )
        self.b_refresh.setText(
            QCoreApplication.translate("DDataPlotter", "Refresh", None)
        )

    # retranslateUi



import numpy as np

from SciDataTool.Functions.filter_spectral_leakage import (
    filter_spectral_leakage as filter_spectral_leakage_fct,
)


def filter_spectral_leakage(
    self,
    freqs_th,
    axes_list_filt=list(),
    arg_list=list(),
    Wmatf=np.array([]),
    If=np.array([]),
    is_return_calc_data=False,
):
    """Filter spectral leakage from a SciDataTool DataND object

    Parameters
    ----------
    self : DataND
        a DataND object
    freqs_th : ndarray
        theoretical frequencies
    axes_list_filt : [Data]
        Axes list to be stored in filtered DataFreq
    arg_list : list
        List of axes arguments to call SciDataTool get_along()
    Wmatf : ndarray
        Filtering matrix
    If: ndarray
        Index of FFT harmonics components used to calculate Wmatf
    is_return_all_args: bool
        True to return all data that can be used to speed calculation for further use

    Returns
    -------
    data_filt : DataND
        a DataND object
    axes_list_filt : [Data]
        Axes list to be stored in filtered DataFreq
    arg_list : list
        List of axes arguments to call SciDataTool get_along()
    Wmatf : ndarray
        Filtering matrix
    If: ndarray
        Index of FFT harmonics components used to calculate Wmatf
    freqs_th : ndarray
        theoretical frequencies
    """

    module = __import__("SciDataTool.Classes.DataFreq", fromlist=["DataFreq"])
    DataFreq = getattr(module, "DataFreq")

    module = __import__("SciDataTool.Classes.DataPattern", fromlist=["DataPattern"])
    DataPattern = getattr(module, "DataPattern")

    module = __import__("SciDataTool.Classes.Data1D", fromlist=["Data1D"])
    Data1D = getattr(module, "Data1D")

    if len(axes_list_filt) == 0 or len(arg_list) == 0:
        # Get fft along frequency axis (keep other axes)
        arg_list = list()  # Reinstantiate a different list
        axes_list = self.get_axes()
        for axis in axes_list:
            if axis.name == "time":
                arg_list.append("freqs")
            elif axis.name == "angle":
                arg_list.append("wavenumber")
            elif axis.name == "z" and isinstance(axis, DataPattern):
                arg_list.append("z[smallestpattern]")
            else:
                arg_list.append(axis.name)

    is_real = self.is_real
    self.is_real = False  # To have negative frequencies
    result = self.get_along(*arg_list, is_squeeze=False)
    freqs = result["freqs"]
    spectrum = result[self.symbol]

    if Wmatf.size == 0:
        # Get time step
        Time = self.get_axes("time")
        if len(Time) > 0:
            time = Time[0].get_values()
        else:
            result_time = self.get_along("time")
            time = result_time["time"]
        dt = time[1] - time[0]
        Nt = time.size
    else:
        Nt, dt = None, None

    if is_real and np.where(freqs < 0)[0].size == 0:
        # Mirror values to negative frequencies
        If0 = freqs != 0
        spectrum[If0, ...] *= 0.5
        If0[-1] = bool(np.mod(freqs.size, 2) == 0)
        freqs = np.concatenate((-np.flip(freqs), freqs[If0]))
        spectrum = np.concatenate(
            (
                np.flip(np.conj(spectrum)),
                spectrum[If0, ...],
            ),
            axis=0,
        )

    # Restrict filtering to negative wavenumber only
    if is_real and "wavenumber" in result:
        wavenumber = result["wavenumber"]
        Irn = wavenumber <= 0
        spectrum = spectrum[:, Irn, ...]

    spectrum_filt, freqs_th, Wmatf, If = filter_spectral_leakage_fct(
        spectrum, freqs, freqs_th, Nt, dt, is_freq_pos=False, Wmatf=Wmatf, If=If
    )

    if is_real:
        # Multiply all components by two except zero frequency component
        spectrum_filt *= 2
        I0 = freqs_th == 0
        spectrum_filt[I0, ...] = 0.5 * spectrum_filt[I0, ...]

        # Keep only positive frequencies
        Ifp = freqs_th >= 0
        freqs_th_store = freqs_th[Ifp]

        # Mirror spectrum
        if "wavenumber" in result:
            # Mirror negative frequencies to negative wavenumbers
            Ifn = ~Ifp
            if freqs_th_store[0] == 0:
                Ifn[I0] = True
            spectrum_filt = np.concatenate(
                (
                    spectrum_filt[Ifp, ...],
                    np.flip(np.conj(spectrum_filt[Ifn, 1:-1, ...])),
                ),
                axis=1,
            )
        else:
            spectrum_filt = spectrum_filt[Ifp, ...]

    else:
        freqs_th_store = np.array(freqs_th)

    if len(axes_list_filt) == 0:
        axes_list_filt = list()  # Reinstantiate a different list
        axes_list_filt.append(
            Data1D(
                name="freqs",
                unit="Hz",
                values=freqs_th_store,
                normalizations=axes_list[0].normalizations,
                symmetries=dict(),
            )
        )

        if "wavenumber" in result:
            axes_list_filt.append(
                Data1D(
                    name="wavenumber",
                    unit="dimless",
                    values=result["wavenumber"],
                    normalizations=axes_list[1].normalizations,
                    symmetries=dict(),
                )
            )

        # Copy other axes
        if "angle" in [axis.name for axis in self.axes] or "wavenumber" in [
            axis.name for axis in self.axes
        ]:
            n = 2
        else:
            n = 1
        for axis in axes_list[n:]:
            axes_list_filt.append(axis.copy())

    data_filt = DataFreq(
        name=self.name,
        symbol=self.symbol,
        unit=self.unit,
        axes=axes_list_filt,
        values=spectrum_filt,
        normalizations=self.normalizations,
        is_real=True,
    )

    self.is_real = is_real

    if is_return_calc_data:
        return data_filt, axes_list_filt, arg_list, Wmatf, If, freqs_th

    else:
        return data_filt



import logging


class GifHandler(logging.Handler):
    def __init__(self, parent):
        super().__init__()
        self.parent = parent

    def emit(self, record):
        """Send signal gif_available to specify the end of the gif generation"""
        self.parent.gif_available.emit()



from numpy import abs as np_abs, nanmax, nanmin

from SciDataTool.Functions.derivation_integration import (
    derivate,
    integrate,
    integrate_local,
    integrate_local_pattern,
    antiderivate,
)
from SciDataTool.Functions.sum_mean import (
    my_sum,
    my_mean,
    root_mean_square,
    root_sum_square,
)


def _apply_operations(self, values, axes_list, is_magnitude, unit, corr_unit):
    """Returns the values of the field transformed or converted
    Parameters
    ----------
    self: Data
        a Data object
    values: ndarray
        array of the field
    Returns
    -------
    values: ndarray
        values of the field
    """

    # Take magnitude before summing
    if is_magnitude and "dB" not in unit:
        values = np_abs(values)

    # Apply sums, means, etc
    for axis_requested in axes_list:
        # Get axis data
        ax_val = axis_requested.values
        extension = axis_requested.extension
        index = axis_requested.index
        if axis_requested.is_pattern:
            Nper, is_aper = None, None
        else:
            Nper, is_aper = self.axes[index].get_periodicity()
        if axis_requested.name in ["time", "angle", "z"]:
            is_phys = True
            is_freqs = False
        elif axis_requested.name in ["freqs", "frequency"]:
            is_phys = False
            is_freqs = True
        else:
            is_phys = False
            is_freqs = False
        if axis_requested.name in ["freqs", "frequency", "wavenumber"]:
            is_fft = True
        else:
            is_fft = False
        # max over max axes
        if extension in "max":
            values = nanmax(values, axis=index)
        # min over max axes
        elif extension in "min":
            values = nanmin(values, axis=index)
        # sum over sum axes
        elif extension in "sum":
            values = my_sum(
                values, index, Nper, is_aper, unit, is_fft, corr_unit=corr_unit
            )
        # root sum square over rss axes
        elif extension == "rss":
            values = root_sum_square(
                values,
                ax_val,
                index,
                Nper,
                is_aper,
                is_phys,
                unit,
                is_fft,
                corr_unit=corr_unit,
            )
        # mean value over mean axes
        elif extension == "mean":
            values = my_mean(values, ax_val, index, Nper, is_aper, is_phys, is_fft)
        # RMS over rms axes
        elif extension == "rms":
            values = root_mean_square(
                values, ax_val, index, Nper, is_aper, is_phys, is_fft
            )
        # integration over integration axes
        elif extension == "integrate":
            values = integrate(values, ax_val, index, Nper, is_aper, is_phys)
        # local integration over integration axes
        elif extension == "integrate_local":
            if axis_requested.name == "z":
                values, ax_val = integrate_local_pattern(values, ax_val, index)
                axis_requested.values = ax_val
            else:
                values = integrate_local(
                    values, ax_val, index, Nper, is_aper, is_phys, is_freqs
                )
        # antiderivation over antiderivation axes
        elif extension == "antiderivate":
            values = antiderivate(
                values, ax_val, index, Nper, is_aper, is_phys, is_freqs
            )
        # derivation over derivation axes
        elif extension == "derivate":
            values = derivate(values, ax_val, index, Nper, is_aper, is_phys, is_freqs)

    if is_magnitude and "dB" in unit:  # Correction for negative/small dB/dBA
        values[values < 2] = 0
        values = np_abs(values)

    return values



def is_freq(*args):
    """Determine if input args require FreqPart"""
    is_freq = False
    if len(args) == 1 and type(args[0]) == tuple:
        args = args[0]  # if called from another script with *args
    for arg in args:
        if "freqs" in arg or "wavenumber" in arg:
            is_freq = True
            break
    return is_freq



from SciDataTool import Data1D
from SciDataTool.Functions import axes_dict, rev_axes_dict
from SciDataTool.Functions.conversions import get_unit_derivate, get_unit_integrate


def get_data_along(self, *args, unit="SI", is_norm=False, axis_data=[]):
    """Returns the sliced or interpolated version of the data, using conversions and symmetries if needed.
    Parameters
    ----------
    self: Data
        a Data object
    *args: list of strings
        List of axes requested by the user, their units and values (optional)
    unit: str
        Unit requested by the user ("SI" by default)
    is_norm: bool
        Boolean indicating if the field must be normalized (False by default)
    axis_data: list
        list of ndarray corresponding to user-input data
    Returns
    -------
    a DataND object
    """

    if "dB" in unit:
        results = self.get_magnitude_along(
            *args, is_squeeze=False, unit=unit, is_norm=is_norm, axis_data=axis_data
        )
    else:
        results = self.get_along(
            *args, is_squeeze=False, unit=unit, is_norm=is_norm, axis_data=axis_data
        )
    values = results.pop(self.symbol)
    del results["axes_dict_other"]
    axes_list = results.pop("axes_list")
    Axes = []

    axes_name_new = list(results.keys())
    if "time" in axes_name_new:
        Data_type = "DataTime"
    elif "freqs" in axes_name_new:
        Data_type = "DataFreq"
    else:
        Data_type = "DataND"

    # Dynamic import to avoid loop
    module = __import__("SciDataTool.Classes." + Data_type, fromlist=[Data_type])
    DataClass = getattr(module, Data_type)

    for axis_name in axes_name_new:
        if not isinstance(results[axis_name], str):
            for i, axis in enumerate(self.axes):
                if axis.name == axis_name:
                    index = i
                    name = axis.name
                    is_components = axis.is_components
                    axis_values = results[axis_name]
                    ax_unit = axis.unit
                elif axis_name in axes_dict:
                    if axes_dict[axis_name][0] == axis.name:
                        index = i
                        name = axis_name
                        is_components = axis.is_components
                        axis_values = results[axis_name]
                        ax_unit = axes_dict[axis_name][2]
                elif axis_name in rev_axes_dict:
                    if rev_axes_dict[axis_name][0] == axis.name:
                        index = i
                        name = axis_name
                        is_components = axis.is_components
                        axis_values = results[axis_name]
                        ax_unit = rev_axes_dict[axis_name][2]
            # Update symmetries
            if (
                "smallestperiod" in args[index]
                or args[index]
                in [
                    "freqs",
                    "wavenumber",
                ]
                or "antiderivate" in args[index]
                or "derivate" in args[index]
            ):
                symmetries = self.axes[index].symmetries.copy()
            else:
                symmetries = dict()
            Axis = Data1D(
                name=name,
                unit=ax_unit,
                values=axis_values,
                is_components=is_components,
                normalizations=self.axes[index].normalizations.copy(),
                symmetries=symmetries,
            ).to_linspace()
            if hasattr(self.axes[index], "sort_indices") and hasattr(
                Axis, "sort_indices"
            ):
                Axis.sort_indices = self.axes[index].sort_indices
            Axes.append(Axis)

    # Update unit if dB/dBA conversion
    if "dB" in unit:
        unit = unit
    else:
        # Update unit if derivation or integration
        unit = self.unit
        for axis in axes_list:
            if axis.extension in ["antiderivate", "integrate", "integrate_local"]:
                unit = get_unit_integrate(unit, axis.corr_unit)
            elif axis.extension == "derivate":
                unit = get_unit_derivate(unit, axis.corr_unit)

    return DataClass(
        name=self.name,
        unit=unit,
        symbol=self.symbol,
        axes=Axes,
        values=values,
        normalizations=self.normalizations.copy(),
        is_real=self.is_real,
    )



from PySide2.QtWidgets import (
    QWidget,
    QStyle,
    QStyledItemDelegate,
    QStyleOptionButton,
    QApplication,
    QCheckBox,
    QHeaderView,
)
from PySide2.QtCore import Qt, QEvent, QPoint, QRect
from PySide2.QtGui import QStandardItemModel, QStandardItem, QPixmap, QIcon
from PySide2.QtCore import Signal, QSortFilterProxyModel, QSize

from numpy import array

from SciDataTool.GUI.WFilter.Ui_WFilter import Ui_WFilter
from SciDataTool.Functions.Plot import axes_dict

# Column id
VALUE_COL = 0
PLOT_COL = 1


class WFilter(Ui_WFilter, QWidget):
    """Widget to select the Data/output range"""

    refreshNeeded = Signal()

    def __init__(self, axis, indices=None, parent=None, path_to_image=None):
        """Linking the button with their method + initializing the arguments used

        Parameters
        ----------
        self : WDataRange
            a WDataRange object
        parent : QWidget
            The parent QWidget
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self, parent=parent)
        self.setupUi(self)

        self.axis = axis
        self.indices = indices
        self.axis_values = self.axis.get_values()
        self.path_to_image = path_to_image
        self.init_table()

        if self.axis.name in axes_dict:
            self.setWindowTitle("Filtering on " + axes_dict[self.axis.name])
        else:
            self.setWindowTitle("Filtering on " + self.axis.name)

        self.cb_all.stateChanged.connect(self.select_all)
        self.b_Ok.clicked.connect(self.update_and_close)
        self.b_cancel.clicked.connect(self.cancel_and_close)

    def select_all(self):
        for i in range(self.tab_indices.model().rowCount()):
            # Check all checkboxes
            self.tab_indices.model().data(
                self.tab_indices.model().index(
                    i, self.tab_indices.model().columnCount() - 1
                )
            ).setCheckState(self.cb_all.checkState())

    def cancel_and_close(self):
        """Method called when the user click on the cancel button
        Parameters
        ----------
        self : WDataRange
            a WDataRange object
        """

        self.close()

    def init_table(self):
        """Method that fill the table with the values of the axis, each line corresponds to one index
        Parameters
        ----------
        self : WDataRange
            a WDataRange object"""
        self.tab_indices.setSortingEnabled(True)

        if self.axis.is_components:
            # If we have an axis with components, then we use the filters to build the complete table
            if hasattr(self.axis, "filter") and self.axis.filter is not None:
                filter_list = list(self.axis.filter.keys())
            else:
                ncol = len(self.axis_values[0].split(self.axis.delimiter))
                filter_list = ["" for i in range(ncol)]
            if (
                hasattr(self.axis, "sort_indices")
                and self.axis.sort_indices is not None
            ):
                filter_list.append("Rank")
            filter_list.append("Plot")  # Adding the column with checkbox

            # Prepare strings
            data = []
            for i, value in enumerate(self.axis_values):
                data_i = []
                elems = value.split(self.axis.delimiter)
                for j, elem in enumerate(elems):
                    string = elem.replace(filter_list[j] + "=", "")
                    if " [" in string:
                        if i == len(self.axis_values) - 1:
                            filter_list[j] = (
                                filter_list[j] + " [" + string.split(" [")[1]
                            ).replace(" (failed)", "")
                        if "failed" in string:
                            string = string.split(" [")[0] + " (failed)"
                        else:
                            string = string.split(" [")[0]
                    if self.axis.char_to_rm is not None:
                        for char in self.axis.char_to_rm:
                            string = string.replace(char, "")
                        try:
                            string = float(string)
                        except:
                            pass
                    data_i.append(string)
                if "Rank" in filter_list:
                    data_i.append(self.axis.sort_indices.index(i) + 1)
                data.append(data_i)

            # Setting up the table
            data_model = MyTableModel(
                data,
                filter_list,
                self,
            )
            for i in range(len(self.axis_values)):
                item = QStandardItem()
                data_model.setItem(i, len(filter_list), item)
            self.tab_indices.setModel(data_model)

            self.tab_indices.setItemDelegateForColumn(
                len(filter_list) - 1,
                CheckBoxDelegate(
                    self, data_model, self.indices, self.path_to_image, self
                ),
            )

        else:
            # Setting up the table
            data_model = MyTableModel(
                array([[format(value, ".4g") for value in self.axis_values]]).T,
                ["Value", "Plot?"],
                self,
            )
            for i in range(len(self.axis_values)):
                item = QStandardItem()
                data_model.setItem(i, 2, item)
            self.tab_indices.setModel(data_model)

            self.tab_indices.setItemDelegateForColumn(
                1,
                CheckBoxDelegate(
                    self, data_model, self.indices, self.path_to_image, self
                ),
            )

        if self.indices is None or len(self.indices) == len(self.axis_values):
            self.cb_all.setChecked(True)

        # Enable sorting
        tableModel = self.tab_indices.model()
        proxyModel = QSortFilterProxyModel()
        proxyModel.setSourceModel(tableModel)
        self.tab_indices.setModel(proxyModel)

        # Adjust column width
        header = self.tab_indices.horizontalHeader()
        for i in range(self.tab_indices.model().columnCount()):
            header.setSectionResizeMode(i, QHeaderView.ResizeToContents)

        # Connect signal
        data_model.cb_edited.connect(self.update_cb_all)

    def update_cb_all(self):
        self.cb_all.blockSignals(True)
        self.cb_all.setCheckState(Qt.PartiallyChecked)
        self.cb_all.blockSignals(False)

    def update_and_close(self):
        """Method called when the click on the Ok button
        Parameters
        ----------
        self : WDataRange
            a WDataRange object
        """

        # Get checked indices
        indices = []
        for i in range(self.tab_indices.model().rowCount()):
            # Get checked indices
            if (
                self.tab_indices.model()
                .data(
                    self.tab_indices.model().index(
                        i, self.tab_indices.model().columnCount() - 1
                    )
                )
                .checkState()
                .__bool__()
            ):  # Use mapping to get indices before sorting
                indices.append(
                    self.tab_indices.model()
                    .mapToSource(self.tab_indices.model().index(i, 0))
                    .row()
                )
        if indices != self.indices:
            self.indices = indices
            self.refreshNeeded.emit()
        self.close()


class MyTableModel(QStandardItemModel):
    cb_edited = Signal()

    def __init__(self, datain, header, parent=None):
        QStandardItemModel.__init__(self, parent)
        self.arraydata = datain
        self.header_labels = header

    def rowCount(self, parent):
        return len(self.arraydata)

    def columnCount(self, parent):
        return len(self.header_labels)

    def data(self, index, role):
        if role == Qt.EditRole:
            return None
        elif role != Qt.DisplayRole:
            return None
        if index.column() == self.columnCount(None) - 1:
            return self.itemFromIndex(index)
        else:
            return self.arraydata[index.row()][index.column()]

    def headerData(self, section, orientation, role=Qt.DisplayRole):
        if role == Qt.DisplayRole and orientation == Qt.Horizontal:
            return self.header_labels[section]
        return QStandardItemModel.headerData(self, section, orientation, role)

    def setData(self, index, value, role):
        if role == Qt.EditRole:
            if index.column() == self.columnCount(None) - 1:
                self.itemFromIndex(index).setCheckState(Qt.CheckState(value))
                self.cb_edited.emit()
        else:
            super(MyTableModel, self).setData(index, value, role)
        return value


class CheckBoxDelegate(QStyledItemDelegate):
    """
    A delegate that places a fully functioning QCheckBox in every
    cell of the column to which it's applied
    """

    def __init__(self, parent, model, indices=None, path_to_image=None, widget=None):
        QStyledItemDelegate.__init__(self, parent)
        self.model = model
        self.widget = widget
        self.path_to_image = path_to_image
        # Check all boxes at initialization
        if indices is None:
            indices = [i for i in range(self.model.rowCount(None))]
        for index in indices:
            self.model.setData(
                self.model.index(index, self.model.columnCount(None) - 1),
                True,
                Qt.EditRole,
            )

    def createEditor(self, parent, option, index):
        """
        Important, otherwise an editor is created if the user clicks in this cell.
        ** Need to hook up a signal to the model
        """
        return None

    def paint(self, painter, option, index):
        """
        Paint a checkbox without the label.
        """

        checked = index.data().checkState().__bool__()
        check_box_style_option = QStyleOptionButton()
        check_box_style_option.rect = self.getCheckBoxRect(option)
        if self.path_to_image is not None:
            icon = QIcon()
            icon.addPixmap(
                QPixmap(self.path_to_image + "/images/icon/checkbox-1-checked.png"),
                QIcon.Normal,
                QIcon.On,
            )
            icon.addPixmap(
                QPixmap(self.path_to_image + "/images/icon/checkbox-1-unchecked.png"),
                QIcon.Normal,
                QIcon.Off,
            )
            check_box_style_option.iconSize = QSize(16, 16)
            check_box_style_option.icon = icon
            check_box_style_option.features = QStyleOptionButton.Flat

        if (index.flags() & Qt.ItemIsEditable) > 0:
            check_box_style_option.state |= QStyle.State_Enabled
        else:
            check_box_style_option.state |= QStyle.State_ReadOnly

        if checked:
            check_box_style_option.state |= QStyle.State_On
        else:
            check_box_style_option.state |= QStyle.State_Off

        # check_box_style_option.state |= QStyle.State_Enabled

        QApplication.style().drawControl(
            QStyle.CE_PushButton, check_box_style_option, painter
        )

    def editorEvent(self, event, model, option, index):
        """
        Change the data in the model and the state of the checkbox
        if the user presses the left mousebutton or presses
        Key_Space or Key_Select and this cell is editable. Otherwise do nothing.
        """
        if not (index.flags() & Qt.ItemIsEditable) > 0:
            return False

        # Do not change the checkbox-state
        if event.type() == QEvent.MouseButtonPress:
            return False
        if (
            event.type() == QEvent.MouseButtonRelease
            or event.type() == QEvent.MouseButtonDblClick
        ):
            if event.button() != Qt.LeftButton or not self.getCheckBoxRect(
                option
            ).contains(event.pos()):
                return False
            if event.type() == QEvent.MouseButtonDblClick:
                return True
        elif event.type() == QEvent.KeyPress:
            if event.key() != Qt.Key_Space and event.key() != Qt.Key_Select:
                return False
        else:
            return False

        # Change the checkbox-state
        self.setModelData(None, model, index)
        return True

    def setModelData(self, editor, model, index):
        """
        The user wanted to change the old state in the opposite.
        """
        newValue = not index.data().checkState()
        model.setData(index, newValue, Qt.EditRole)

    def getCheckBoxRect(self, option):
        check_box_style_option = QStyleOptionButton()
        check_box_rect = QApplication.style().subElementRect(
            QStyle.SE_CheckBoxIndicator, check_box_style_option, None
        )
        check_box_point = QPoint(
            option.rect.x() + option.rect.width() / 2 - check_box_rect.width() / 2,
            option.rect.y() + option.rect.height() / 2 - check_box_rect.height() / 2,
        )
        return QRect(check_box_point, check_box_rect.size())



from SciDataTool.Functions import AxisError, NormError, UnitError
from SciDataTool.Functions.conversions import convert, to_dB, to_dBA, to_noct
from SciDataTool.Functions.fix_axes_order import fix_axes_order
from numpy import apply_along_axis, add, take


def get_magnitude_along(
    self,
    *args,
    unit="SI",
    is_norm=False,
    axis_data=[],
    is_squeeze=True,
    is_sum=True,
    corr_unit=None
):
    """Returns the ndarray of the magnitude of the FT, using conversions and symmetries if needed.
    Parameters
    ----------
    self: Data
        a Data object
    *args: list of strings
        List of axes requested by the user, their units and values (optional)
    unit: str
        Unit requested by the user ("SI" by default)
    is_norm: bool
        Boolean indicating if the field must be normalized (False by default)
    axis_data: list
        list of ndarray corresponding to user-input data
    Returns
    -------
    list of 1Darray of axes values, ndarray of magnitude values
    """
    if len(args) == 1 and type(args[0]) == tuple:
        args = args[0]  # if called from another script with *args

    # Fix axes order
    args = fix_axes_order([axis.name for axis in self.get_axes()], args)

    # For dB/dBA conversions, first extraction with freqs axis
    if "dB" in unit and unit != self.unit and is_sum:
        new_args = list(args).copy()
        freq_name = None
        index_freq = None
        index_speed = None
        index_order = None
        args_freqs = ["freqs", "frequency", "time"]
        for i, axis in enumerate(self.axes):  # Find frequency axis
            if axis.name in args_freqs:
                freq_name = axis.name
        if freq_name is not None:
            for i, arg in enumerate(args):
                if freq_name in arg:
                    if "sum" in arg or "rss" in arg:
                        new_args[i] = freq_name
                        index_freq = i
            if index_freq is None:  # No sum on freqs axis -> can extract directly
                return self.get_magnitude_along(
                    *args,
                    axis_data=axis_data,
                    unit=unit,
                    is_squeeze=is_squeeze,
                    is_sum=False,
                )
            else:
                data = self.get_data_along(
                    *new_args, axis_data=axis_data, unit=unit
                )  # Extract first along freqs axis
                return data.get_magnitude_along(
                    *[args[index_freq]],
                    axis_data=axis_data,
                    unit=unit,
                    is_squeeze=is_squeeze,
                    is_sum=False,
                    corr_unit=self.unit,
                )  # Then sum on freqs axis
        else:  # Try speed/order
            is_match = 0
            for i, axis in enumerate(self.axes):  # Find frequency axis
                if axis.name == "speed":
                    is_match += 1
                    arg_speed = "speed"
                elif axis.name == "order":
                    is_match += 1
                elif "speed" in axis.normalizations:
                    is_match += 1
                    arg_speed = axis.name + "->speed"
            if is_match == 2:
                for i, arg in enumerate(args):
                    if "speed" in arg:
                        if arg.split("{")[0] != arg_speed:
                            new_args[i] = arg_speed
                            index_speed = i
                    elif "order" in arg:
                        if arg.split("{")[0] != "order":
                            new_args[i] = "order"
                            index_order = i
            elif is_match == 1:
                for i, arg in enumerate(args):
                    if "speed" in arg:
                        if arg.split("{")[0] != arg_speed:
                            new_args[i] = arg_speed
                            index_speed = i
            elif unit == "dB" or "A-weight" in self.normalizations:
                self.get_magnitude_along(
                    *args,
                    axis_data=axis_data,
                    unit=unit,
                    is_squeeze=is_squeeze,
                    is_sum=False,
                )
            else:
                raise AxisError(
                    "Cannot convert to " + unit + " without a frequency axis"
                )
            if (
                index_speed is None and index_order is None
            ):  # No sum on freqs axis -> can extract directly
                return self.get_magnitude_along(
                    *args,
                    axis_data=axis_data,
                    unit=unit,
                    is_squeeze=is_squeeze,
                    is_sum=False,
                )
            elif index_speed is None:  # Sum on order axis
                data = self.get_data_along(
                    *new_args, axis_data=axis_data, unit=unit
                )  # Extract first along order axis
                return data.get_magnitude_along(
                    *[arg_speed, args[index_order]],
                    axis_data=axis_data,
                    unit=unit,
                    is_squeeze=is_squeeze,
                    is_sum=False,
                    corr_unit=self.unit,
                )  # Then sum on order axis
            elif index_order is None:  # Sum on speed axis
                data = self.get_data_along(
                    *new_args, axis_data=axis_data, unit=unit
                )  # Extract first along speed axis
                return data.get_magnitude_along(
                    *args,
                    axis_data=axis_data,
                    unit=unit,
                    is_squeeze=is_squeeze,
                    is_sum=False,
                    corr_unit=self.unit,
                )  # Then sum on speed axis
            else:  # Sum on speed and order axes
                data = self.get_data_along(
                    *new_args, axis_data=axis_data, unit=unit
                )  # Extract first along speed and order axes
                return data.get_magnitude_along(
                    *[args[index_speed], args[index_order]],
                    axis_data=axis_data,
                    unit=unit,
                    is_squeeze=is_squeeze,
                    is_sum=False,
                    corr_unit=self.unit,
                )  # Then sum on speed and order axes

    else:

        return_dict = self.get_along(
            *args,
            axis_data=axis_data,
            is_squeeze=is_squeeze,
            is_magnitude=True,
            corr_unit=corr_unit,
        )
        values = return_dict[self.symbol]

        # 1/nth octave band
        for axis in return_dict["axes_list"]:
            if axis.name == "freqs" or axis.corr_name == "freqs":
                index = axis.index
                if axis.noct is not None:
                    (values, foct) = apply_along_axis(
                        to_noct, index, values, return_dict["freqs"], noct=axis.noct
                    )
                    return_dict[axis.name] = foct
        # Convert into right unit (apart because of dBA conversion)
        if unit == self.unit or unit == "SI":
            if is_norm:
                try:
                    values = self.normalizations["ref"].normalize(values)
                except:
                    raise NormError("Reference value not specified for normalization")
        elif unit == "dB":
            ref_value = 1.0
            if "ref" in self.normalizations:
                ref_value *= self.normalizations["ref"].ref
            values = to_dB(values, self.unit, ref_value)
        elif unit == "dBA":
            ref_value = 1.0
            if "ref" in self.normalizations:
                ref_value *= self.normalizations["ref"].ref
            if "A-weight" in self.normalizations:
                values = to_dB(values, self.unit, ref_value)
                # Extract requested indices from A-weight
                axes_list = return_dict["axes_list"]
                index_list = []
                for ii, axis in enumerate(axes_list):
                    if axis.indices is not None and len(axis.indices) > 1:
                        index_list.append(ii)
                A_weight = self.normalizations["A-weight"].vector
                for index in index_list:
                    A_weight = take(
                        A_weight,
                        axes_list[index].indices,
                        axis=index,
                    )
                # Flatten to apply A-weighting
                A_weight = A_weight.ravel("C")
                shape = values.shape
                values = values.reshape(
                    A_weight.shape
                    + shape[len(self.normalizations["A-weight"].vector.shape) :]
                )
                values = apply_along_axis(add, 0, values, A_weight)
                values = values.reshape(shape)
            elif "freqs" in return_dict.keys():
                for axis in return_dict["axes_list"]:
                    if axis.name == "freqs" or axis.corr_name == "freqs":
                        index = axis.index
                else:
                    if return_dict["axes_list"][
                        index
                    ].corr_values is not None and return_dict["axes_list"][
                        index
                    ].unit not in [
                        "SI",
                        return_dict["axes_list"][index].corr_unit,
                    ]:
                        freqs = return_dict["axes_list"][index].corr_values
                    else:
                        freqs = return_dict["freqs"]
                values = apply_along_axis(
                    to_dBA, index, values, freqs, self.unit, ref_value
                )
            elif "frequency" in return_dict.keys():
                for axis in return_dict["axes_list"]:
                    if axis.name == "frequency" or axis.corr_name == "frequency":
                        index = axis.index
                else:
                    if return_dict["axes_list"][
                        index
                    ].corr_values is not None and return_dict["axes_list"][
                        index
                    ].unit not in [
                        "SI",
                        return_dict["axes_list"][index].corr_unit,
                    ]:
                        freqs = return_dict["axes_list"][index].corr_values
                    else:
                        freqs = return_dict["frequency"]
                values = apply_along_axis(
                    to_dBA, index, values, freqs, self.unit, ref_value
                )
            elif "order" in return_dict:
                freqs = self._get_freqs()
                freqs = freqs.ravel("C")
                shape = values.shape
                values = values.reshape(freqs.shape + shape[2:])
                values = apply_along_axis(
                    to_dBA, 0, values, freqs, self.unit, ref_value
                )
                values = values.reshape(shape)
            else:
                raise UnitError(
                    "dBA conversion only available for fft with frequencies"
                )

        elif unit in self.normalizations:
            values = self.normalizations.get(unit).normalize(values)
        else:
            values = convert(values, self.unit, unit)
        return_dict[self.symbol] = values
        return return_dict



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/DataDual.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/SciDataTool/tree/master/SciDataTool/Methods//DataDual
"""

from os import linesep
from sys import getsizeof
from ._check import set_array, check_var, raise_
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .DataND import DataND

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.DataDual.get_along import get_along
except ImportError as error:
    get_along = error

try:
    from ..Methods.DataDual.get_axes import get_axes
except ImportError as error:
    get_axes = error


from numpy import array, array_equal
from ._check import InitUnKnowClassError


class DataDual(DataND):
    """Class for fields defined in both time and freq domains"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.DataDual.get_along
    if isinstance(get_along, ImportError):
        get_along = property(
            fget=lambda x: raise_(
                ImportError("Can't use DataDual method get_along: " + str(get_along))
            )
        )
    else:
        get_along = get_along
    # cf Methods.DataDual.get_axes
    if isinstance(get_axes, ImportError):
        get_axes = property(
            fget=lambda x: raise_(
                ImportError("Can't use DataDual method get_axes: " + str(get_axes))
            )
        )
    else:
        get_axes = get_axes
    # save and copy methods are available in all object
    save = save
    copy = copy

    def __init__(
        self,
        axes_dt=None,
        values_dt=None,
        axes_df=None,
        values_df=None,
        axes=None,
        FTparameters=-1,
        values=None,
        is_real=True,
        symbol="",
        name="",
        unit="",
        normalizations=-1,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for SciDataTool type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for SciDataTool Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "axes_dt" in list(init_dict.keys()):
                axes_dt = init_dict["axes_dt"]
            if "values_dt" in list(init_dict.keys()):
                values_dt = init_dict["values_dt"]
            if "axes_df" in list(init_dict.keys()):
                axes_df = init_dict["axes_df"]
            if "values_df" in list(init_dict.keys()):
                values_df = init_dict["values_df"]
            if "axes" in list(init_dict.keys()):
                axes = init_dict["axes"]
            if "FTparameters" in list(init_dict.keys()):
                FTparameters = init_dict["FTparameters"]
            if "values" in list(init_dict.keys()):
                values = init_dict["values"]
            if "is_real" in list(init_dict.keys()):
                is_real = init_dict["is_real"]
            if "symbol" in list(init_dict.keys()):
                symbol = init_dict["symbol"]
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "unit" in list(init_dict.keys()):
                unit = init_dict["unit"]
            if "normalizations" in list(init_dict.keys()):
                normalizations = init_dict["normalizations"]
        # Set the properties (value check and convertion are done in setter)
        self.axes_dt = axes_dt
        self.values_dt = values_dt
        self.axes_df = axes_df
        self.values_df = values_df
        # Call DataND init
        super(DataDual, self).__init__(
            axes=axes,
            FTparameters=FTparameters,
            values=values,
            is_real=is_real,
            symbol=symbol,
            name=name,
            unit=unit,
            normalizations=normalizations,
        )
        # The class is frozen (in DataND init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        DataDual_str = ""
        # Get the properties inherited from DataND
        DataDual_str += super(DataDual, self).__str__()
        DataDual_str += "axes_dt = " + str(self.axes_dt) + linesep + linesep
        DataDual_str += (
            "values_dt = "
            + linesep
            + str(self.values_dt).replace(linesep, linesep + "\t")
            + linesep
            + linesep
        )
        DataDual_str += "axes_df = " + str(self.axes_df) + linesep + linesep
        DataDual_str += (
            "values_df = "
            + linesep
            + str(self.values_df).replace(linesep, linesep + "\t")
            + linesep
            + linesep
        )
        return DataDual_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from DataND
        if not super(DataDual, self).__eq__(other):
            return False
        if other.axes_dt != self.axes_dt:
            return False
        if not array_equal(other.values_dt, self.values_dt):
            return False
        if other.axes_df != self.axes_df:
            return False
        if not array_equal(other.values_df, self.values_df):
            return False
        return True

    def compare(self, other, name="self", ignore_list=None):
        """Compare two objects and return list of differences"""

        if ignore_list is None:
            ignore_list = list()
        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from DataND
        diff_list.extend(super(DataDual, self).compare(other, name=name))
        if (other.axes_dt is None and self.axes_dt is not None) or (
            other.axes_dt is not None and self.axes_dt is None
        ):
            diff_list.append(name + ".axes_dt None mismatch")
        elif self.axes_dt is None:
            pass
        elif len(other.axes_dt) != len(self.axes_dt):
            diff_list.append("len(" + name + ".axes_dt)")
        else:
            for ii in range(len(other.axes_dt)):
                diff_list.extend(
                    self.axes_dt[ii].compare(
                        other.axes_dt[ii], name=name + ".axes_dt[" + str(ii) + "]"
                    )
                )
        if not array_equal(other.values_dt, self.values_dt):
            diff_list.append(name + ".values_dt")
        if (other.axes_df is None and self.axes_df is not None) or (
            other.axes_df is not None and self.axes_df is None
        ):
            diff_list.append(name + ".axes_df None mismatch")
        elif self.axes_df is None:
            pass
        elif len(other.axes_df) != len(self.axes_df):
            diff_list.append("len(" + name + ".axes_df)")
        else:
            for ii in range(len(other.axes_df)):
                diff_list.extend(
                    self.axes_df[ii].compare(
                        other.axes_df[ii], name=name + ".axes_df[" + str(ii) + "]"
                    )
                )
        if not array_equal(other.values_df, self.values_df):
            diff_list.append(name + ".values_df")
        # Filter ignore differences
        diff_list = list(filter(lambda x: x not in ignore_list, diff_list))
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from DataND
        S += super(DataDual, self).__sizeof__()
        if self.axes_dt is not None:
            for value in self.axes_dt:
                S += getsizeof(value)
        S += getsizeof(self.values_dt)
        if self.axes_df is not None:
            for value in self.axes_df:
                S += getsizeof(value)
        S += getsizeof(self.values_df)
        return S

    def as_dict(self, type_handle_ndarray=0, keep_function=False, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        type_handle_ndarray: int
            How to handle ndarray (0: tolist, 1: copy, 2: nothing)
        keep_function : bool
            True to keep the function object, else return str
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from DataND
        DataDual_dict = super(DataDual, self).as_dict(
            type_handle_ndarray=type_handle_ndarray,
            keep_function=keep_function,
            **kwargs
        )
        if self.axes_dt is None:
            DataDual_dict["axes_dt"] = None
        else:
            DataDual_dict["axes_dt"] = list()
            for obj in self.axes_dt:
                if obj is not None:
                    DataDual_dict["axes_dt"].append(
                        obj.as_dict(
                            type_handle_ndarray=type_handle_ndarray,
                            keep_function=keep_function,
                            **kwargs
                        )
                    )
                else:
                    DataDual_dict["axes_dt"].append(None)
        if self.values_dt is None:
            DataDual_dict["values_dt"] = None
        else:
            if type_handle_ndarray == 0:
                DataDual_dict["values_dt"] = self.values_dt.tolist()
            elif type_handle_ndarray == 1:
                DataDual_dict["values_dt"] = self.values_dt.copy()
            elif type_handle_ndarray == 2:
                DataDual_dict["values_dt"] = self.values_dt
            else:
                raise Exception(
                    "Unknown type_handle_ndarray: " + str(type_handle_ndarray)
                )
        if self.axes_df is None:
            DataDual_dict["axes_df"] = None
        else:
            DataDual_dict["axes_df"] = list()
            for obj in self.axes_df:
                if obj is not None:
                    DataDual_dict["axes_df"].append(
                        obj.as_dict(
                            type_handle_ndarray=type_handle_ndarray,
                            keep_function=keep_function,
                            **kwargs
                        )
                    )
                else:
                    DataDual_dict["axes_df"].append(None)
        if self.values_df is None:
            DataDual_dict["values_df"] = None
        else:
            if type_handle_ndarray == 0:
                DataDual_dict["values_df"] = self.values_df.tolist()
            elif type_handle_ndarray == 1:
                DataDual_dict["values_df"] = self.values_df.copy()
            elif type_handle_ndarray == 2:
                DataDual_dict["values_df"] = self.values_df
            else:
                raise Exception(
                    "Unknown type_handle_ndarray: " + str(type_handle_ndarray)
                )
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        DataDual_dict["__class__"] = "DataDual"
        return DataDual_dict

    def _set_None(self):
        """Set all the properties to None (except SciDataTool object)"""

        self.axes_dt = None
        self.values_dt = None
        self.axes_df = None
        self.values_df = None
        # Set to None the properties inherited from DataND
        super(DataDual, self)._set_None()

    def _get_axes_dt(self):
        """getter of axes_dt"""
        if self._axes_dt is not None:
            for obj in self._axes_dt:
                if obj is not None:
                    obj.parent = self
        return self._axes_dt

    def _set_axes_dt(self, value):
        """setter of axes_dt"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "SciDataTool.Classes", obj.get("__class__"), "axes_dt"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("axes_dt", value, "[Data]")
        self._axes_dt = value

    axes_dt = property(
        fget=_get_axes_dt,
        fset=_set_axes_dt,
        doc=u"""List of the Data1D objects corresponding to the axes in time/space domain

        :Type: [SciDataTool.Classes.Data]
        """,
    )

    def _get_values_dt(self):
        """getter of values_dt"""
        return self._values_dt

    def _set_values_dt(self, value):
        """setter of values_dt"""
        if type(value) is int and value == -1:
            value = array([])
        elif type(value) is list:
            try:
                value = array(value)
            except:
                pass
        check_var("values_dt", value, "ndarray")
        self._values_dt = value

    values_dt = property(
        fget=_get_values_dt,
        fset=_set_values_dt,
        doc=u"""Values of the field in time/space domain

        :Type: ndarray
        """,
    )

    def _get_axes_df(self):
        """getter of axes_df"""
        if self._axes_df is not None:
            for obj in self._axes_df:
                if obj is not None:
                    obj.parent = self
        return self._axes_df

    def _set_axes_df(self, value):
        """setter of axes_df"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "SciDataTool.Classes", obj.get("__class__"), "axes_df"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("axes_df", value, "[Data]")
        self._axes_df = value

    axes_df = property(
        fget=_get_axes_df,
        fset=_set_axes_df,
        doc=u"""List of the Data1D objects corresponding to the axes in fourier domain

        :Type: [SciDataTool.Classes.Data]
        """,
    )

    def _get_values_df(self):
        """getter of values_df"""
        return self._values_df

    def _set_values_df(self, value):
        """setter of values_df"""
        if type(value) is int and value == -1:
            value = array([])
        elif type(value) is list:
            try:
                value = array(value)
            except:
                pass
        check_var("values_df", value, "ndarray")
        self._values_df = value

    values_df = property(
        fget=_get_values_df,
        fset=_set_values_df,
        doc=u"""Values of the field in fourier domain

        :Type: ndarray
        """,
    )



import numpy as np

from SciDataTool.Functions import NormError, UnitError
from SciDataTool.Functions.conversions import convert as convert_unit, to_dB, to_dBA


def _convert(self, values, unit, is_norm, is_squeeze, axes_list):
    """Returns the values of the field transformed or converted.
    Parameters
    ----------
    self: Data
        a Data object
    values: ndarray
        array of the field
    unit: str
        Unit requested by the user ("SI" by default)
    is_norm: bool
        Boolean indicating if the field must be normalized (False by default)
    Returns
    -------
    values: ndarray
        values of the field
    """

    if is_squeeze:
        values = np.squeeze(values)

    if unit == self.unit or unit == "SI":
        if is_norm:
            try:
                values = self.normalizations["ref"].normalize(values)
            except Exception:
                raise NormError("Reference value not specified for normalization")
    elif unit == "dB":
        ref_value = 1.0
        if "ref" in self.normalizations:
            ref_value *= self.normalizations["ref"].ref
        values = to_dB(np.abs(values), self.unit, ref_value)
    elif unit == "dBA":
        ref_value = 1.0
        if "ref" in self.normalizations:
            ref_value *= self.normalizations["ref"].ref
        is_match = False
        for axis in axes_list:
            if axis.name == "freqs" or axis.corr_name == "freqs":
                if axis.corr_values is not None and axis.unit not in [
                    "SI",
                    axis.corr_unit,
                ]:
                    axis_values = axis.corr_values
                else:
                    axis_values = axis.values
                index = axis.index
                values = np.apply_along_axis(
                    to_dBA, index, values, axis_values, self.unit, ref_value
                )
                is_match = True
            elif axis.name == "frequency":
                if axis.corr_values is None:
                    axis_values = axis.values
                else:
                    axis_values = axis.corr_values
                index = axis.index
                values = np.apply_along_axis(
                    to_dBA, index, values, axis_values, self.unit, ref_value
                )
                is_match = True
        if not is_match:
            axis_names = [axis.name for axis in self.axes]
            if "speed" in axis_names and "order" in axis_names:
                freqs = self._get_freqs()
                freqs = freqs.ravel("C")
                shape = values.shape
                values = values.reshape(freqs.shape + shape[2:])
                values = np.apply_along_axis(
                    to_dBA, 0, values, freqs, self.unit, ref_value
                )
                values = values.reshape(shape)
            else:
                raise UnitError(
                    "dBA conversion only available for fft with frequencies"
                )
    elif unit in self.normalizations:
        values = self.normalizations.get(unit).normalize(values)
    else:
        values = convert_unit(values, self.unit, unit)

    return values



from PySide2.QtWidgets import QWidget, QFileDialog, QMessageBox, QVBoxLayout
from PySide2.QtCore import Signal
from PySide2.QtCore import QByteArray, Qt
from PySide2.QtCore import QThread
from PySide2.QtGui import QMovie
from PySide2.QtWidgets import (
    QFileDialog,
    QLabel,
    QSizePolicy,
    QMessageBox,
)

from os.path import dirname, basename, join, isfile
from SciDataTool.Functions.Load.import_class import import_class
from SciDataTool.Functions.is_axes_in_order import is_axes_in_order
from SciDataTool.GUI.Tools.SaveGifWorker import SaveGifWorker
import matplotlib.pyplot as plt

from SciDataTool.GUI.WPlotManager.Ui_WPlotManager import Ui_WPlotManager

SYMBOL_DICT = {
    "time": "t",
    "angle": "\\alpha",
    "axial direction": "z",
    "frequency": "f",
    "wavenumber": "r",
    "rotation speed": "N0",
    "stator current along d-axis": "I_d",
    "stator current along q-axis": "I_q",
    "speed": "N0",
    "torque": "T",
    "a-weighted sound power level": "ASWL",
    "velocity level": "V",
    "reference torque": "T_{ref}",
}


def latex(string):
    """format a string for latex"""
    if "_" in string or "^" in string or "\\" in string:
        string = r"$" + string + "$"
    return string


class WPlotManager(Ui_WPlotManager, QWidget):
    """Main widget of the SciDataTool UI"""

    updatePlot = Signal()
    updatePlotForced = Signal()

    def __init__(self, parent=None):
        """Initialize the widget by linking buttons to methods

        Parameters
        ----------
        self : WPlotManager
            a WPlotManager object
        parent : QWidget
            The parent widget
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self, parent=parent)
        self.setupUi(self)

        self.default_file_path = None  # Path to export in csv
        self.param_dict = dict()  # Dict with param for animation to animation
        module = __import__("SciDataTool")
        DATA_DIR = getattr(module, "DATA_DIR")
        self.save_path = DATA_DIR  # Path to directory where animation are stored
        self.path_to_image = None  # Path to recover the image for the animate button
        self.main_widget = None

        self.is_test = False  # Used in test to disable showing the animation
        self.gif_path_list = list()  # List of path to the gifs created (used in test)
        self.logger = None

        # Storing each animation as a list with a QMovie, a QLabel and the path to the gif inside gif_widget_list
        self.gif_widget_list = list()

        # Building the interaction with the UI and the UI itself
        self.w_axis_manager.refreshRange.connect(self.update_range)
        self.b_export.clicked.connect(self.export)

        # Linking the signals for the autoRefresh
        self.w_axis_manager.refreshNeeded.connect(self.auto_update)
        self.w_axis_manager.generateAnimation.connect(self.gen_animate)
        self.w_range.refreshNeeded.connect(self.auto_update)

    def auto_update(self):
        """Method that update range before sending the signal to update the plot. The auto-refresh policy will be handled in the DDataPlotter
        Parameters
        ----------
        self : WPlotManager
            a WPlotManager object

        """
        self.updatePlot.emit()

    def close_gif(self, ev, animation_label_closed):
        """Stops the gif and closes the pop up running"""

        for idx in range(len(self.gif_widget_list)):

            animation_label = self.gif_widget_list[idx][1]

            if animation_label == animation_label_closed:
                gif_widget = self.gif_widget_list[idx][0]
                widget = self.gif_widget_list[idx][3]
                break

        # Stopping the animation and resetting the gif widget (QMovie)
        gif_widget.stop()
        gif_widget.setParent(None)
        gif_widget = None
        # Closing and resetting to None the windows that pops up to show animation
        animation_label.close()
        animation_label.setParent(None)
        animation_label = None

        # Closing the widget containing the animation
        widget.close()
        widget.setParent(None)
        widget = None

        # Removing the animation from the list
        self.gif_widget_list.pop(idx)

    def close_all_gif(self):
        """Method used to close all the gif currently running. Only used in test for now"""

        for _, _, _, widget in self.gif_widget_list:
            widget.close()

    def display_gif(self, gif):
        "Method that create a display an animation and store it inside gif_widget_list"

        # Creating widget and layout that will contain the animation and its path
        widget = QWidget()
        layout = QVBoxLayout()

        # Adding the animation (QMovie inside a QLabel) inside the widget
        new_gif_widget = QMovie(gif, QByteArray(), self)
        new_gif_widget.setCacheMode(QMovie.CacheAll)
        new_gif_widget.setSpeed(100)
        # set up the movie screen on a label
        new_animation_label = QLabel(self, Qt.Window)

        # expand and center the label
        new_animation_label.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        new_animation_label.setAlignment(Qt.AlignCenter)
        new_animation_label.setMovie(new_gif_widget)
        new_gif_widget.start()

        layout.addWidget(new_animation_label)

        # Setting size of the widget showing the animation to the size of the gif
        widget.resize(new_gif_widget.currentImage().size())

        # Setting the rest of the widget and showing it
        widget.closeEvent = lambda ev: self.close_gif(ev, new_animation_label)
        widget.setWindowTitle(gif.split("/")[-1].replace(".gif", ""))
        widget.setLayout(layout)
        if not self.is_test:
            widget.show()

        # Hiding the "Generating..." label
        self.l_loading.setHidden(True)

        # Adding the animation to the list
        self.gif_widget_list.append([new_gif_widget, new_animation_label, gif, widget])

    def gen_animate(self):
        """Methods called after clicking on animate button to generate a gif on the axis selected and display it
        Parameters
        ----------
        self : WPlotManager
            a WPlotManager object

        Output
        ---------
        None
        """
        # Recovering axes and operations selected
        axes_selected = self.w_axis_manager.get_axes_selected()
        operations_selected = self.w_axis_manager.get_operation_selected()

        is_3D = len(axes_selected) == 2

        if is_3D:
            not_in_order, axes_selected = is_axes_in_order(axes_selected, self.data)
            self.param_dict["is_switch_axes"] = not_in_order

        # Recovering the axis to animate (operation axis with "to_animate")
        for idx_ope in range(len(operations_selected)):
            ope = operations_selected[idx_ope]
            if "to_animate" in ope:
                animated_axis = ope.split("to_animate")[0]
                idx_animate = idx_ope

        operations_selected.pop(idx_animate)

        # Gathering the other axes and operations selected
        plot_input = [animated_axis]
        plot_input.extend(axes_selected)
        plot_input.extend(operations_selected)

        str_format = ".gif"

        # Updating the figure to make sure that we are recovering the limit of the right figure
        self.update_plot_forced()
        fig = plt.gcf()
        if is_3D and len(plt.gcf().axes) == 2:
            # If we are animating a 3D plot, then we must keep the axes limit

            self.param_dict["x_min"] = fig.axes[0].get_xlim()[0]
            self.param_dict["x_max"] = fig.axes[0].get_xlim()[1]
            self.param_dict["y_min"] = fig.axes[0].get_ylim()[0]
            self.param_dict["y_max"] = fig.axes[0].get_ylim()[1]
            self.param_dict["z_min"] = fig.axes[1].dataLim.extents[0]
            self.param_dict["z_max"] = fig.axes[1].dataLim.extents[-1]

        elif not is_3D:

            if (
                len(axes_selected) == 1
                and axes_selected[0].split("{")[0]
                in [
                    "freqs",
                    "wavenumber",
                ]
                and len(plt.gcf().axes) == 1
            ):
                # if we are animating an FFT plot then we set the limit according to the current figure
                self.param_dict["x_min"] = fig.axes[0].get_xlim()[0]
                self.param_dict["x_max"] = fig.axes[0].get_xlim()[1]
                self.param_dict["y_min"] = fig.axes[0].get_ylim()[0]
                self.param_dict["y_max"] = fig.axes[0].get_ylim()[1]

            else:
                # if we animate a regular 2D plot then we set the limit on y-axis later
                self.param_dict["y_min"] = None
                self.param_dict["y_max"] = None

        # Recovering the suptitle of the figure and adding to the animation
        if fig._suptitle is not None:
            suptitle_ref = fig._suptitle._text
        else:
            suptitle_ref = ""

        # Recovering the name of the gif if not already given
        # if self.default_file_path is None:
        gif_name = self.get_file_name()

        # Recovering "Generating label" from the WSliceOperator with the axis that we want to animate
        for wid in self.w_axis_manager.w_slice_op:
            if wid.axis_name == animated_axis.split("[")[0]:
                self.l_loading = wid.l_loading

        gif = self.save_path + "/" + gif_name + str_format
        gif = gif.replace("\\", "/")

        # Using an index to make sure that we are generating a new gif everytime
        idx = 1
        while gif in self.gif_path_list:
            gif = gif.split(".")[0].split("(")[0] + "(" + str(idx) + ")" + str_format
            idx += 1

        self.gif_path_list.append(gif)

        # Indicating the path to the .gif in the console
        if self.logger is not None:
            self.logger.info("Gif stored at: " + gif)
        else:
            print("Gif stored at: " + gif)

        # Generating a new animation each time
        # Creating a QThread associated to the worker saving the gif
        self.th = QThread(parent=self)
        self.worker = SaveGifWorker(
            widget=self,
            main_widget=self.main_widget,
            gif=gif,
            plot_input=plot_input,
            data_selection=operations_selected,
            is_3D=is_3D,
            suptitle_ref=suptitle_ref,
        )
        self.worker.moveToThread(self.th)

        # Connecting the end of generation of GIF to display, end thread and killing process
        self.worker.gif_available.connect(self.th.finished)
        self.worker.gif_available.connect(lambda: self.worker.kill_worker())
        self.worker.gif_available.connect(lambda: self.display_gif(gif))

        self.th.started.connect(self.worker.run)
        self.th.finished.connect(self.th.quit)
        self.th.start()

        # Showing "Generating..." under the animate button
        self.l_loading.setHidden(False)

    def get_file_name(self):
        """Method that create the name of the file with the name of the field selected"""

        file_name = self.data.name
        file_name = file_name.replace("{", "[").replace("}", "]").replace(".", ",")

        return file_name

    def export(self, save_file_path=False):
        """Method that export the plot as a csv file
        Parameters
        ----------
        self : WPlotManager
            a WPlotManager object
        """
        # Getting the inputs of the user to export the plot + for the name of the csv file
        param_list = [
            *self.w_axis_manager.get_axes_selected(),
            *self.w_axis_manager.get_operation_selected(),
        ]

        if self.default_file_path is None:
            file_name = self.get_file_name()
            default_file_path = file_name + ".csv"
        else:
            default_file_path = self.default_file_path + ".csv"

        # Opening a dialog window to select the directory where the file will be saved if we are not testing
        if save_file_path == False:
            save_file_path = QFileDialog.getSaveFileName(
                self,
                self.tr("Export plot data"),
                default_file_path,
                filter="csv (*.csv)",
            )[0]
        else:
            save_file_path += "\\" + default_file_path

        # Exporting the file to the right folder
        if save_file_path not in ["", False]:
            save_path = dirname(save_file_path)
            file_name = basename(save_file_path).split(".")[0]
            try:
                self.data.export_along(
                    *param_list, save_path=save_path, file_name=file_name
                )
            except Exception as e:
                # Displaying the error inside  abox instead of the console
                err_msg = "Error while exporting Data:\n" + str(e)

                QMessageBox().critical(
                    self,
                    self.tr("Error"),
                    err_msg,
                )

    def get_plot_info(self):
        """Method that gather all the information necessary to plot the new graph.
        Parameters
        ----------
        self : WPlotManager
            a WPlotManager object

        Output
        ----------
        data : DataND
            DataND object that we want to plot
        axes_selected : list
            a list of string corresponding to the axis/axes of the new plot
        data_selection : list
            a list of string corresponding to the operations on the remaining axes for the new plot
        output_range : dict
            a dictionnary with all the info related to WDataRange for the new plot
        """

        # Recovering the axis selected and their units
        axes_selected = self.w_axis_manager.get_axes_selected()

        # Recovering the operation on the other axes
        data_selection = self.w_axis_manager.get_operation_selected()

        # Recovering the operation on the field values
        output_range = self.w_range.get_field_selected()

        return self.data, axes_selected, data_selection, output_range

    def set_info(
        self,
        data,
        axes_request_list=list(),
        component=None,
        unit=None,
        z_min=None,
        z_max=None,
        frozen_type=0,
        is_keep_config=False,
        is_quiver=False,
        plot_arg_dict=dict(),
        save_path="",
        logger=None,
        path_to_image=None,
        main_widget=None,
    ):
        """Method that use the info given by DDataPlotter to setup the widget

        Parameters
        ----------
        self : WPlotManager
            a WPlotManager object
        data : DataND or VectorField object
            A DataND/VectorField object to plot
        axes_request_list:
            list of RequestedAxis which are the info given for the autoplot (for the axes and DataSelection)
        component : str
            Name of the component to plot (For VectorField only)
        unit : str
            unit in which to plot the field
        z_min : float
            Minimum value for Z axis (or Y if only one axe)
        z_max : float
            Minimum value for Z axis (or Y if only one axe)
        frozen_type : int
            0 to let the user modify the axis of the plot, 1 to let him switch them, 2 to not let him change them,
            3 to freeze both axes and operations, 4 to freeze fft, 5 to only allow switch
        plot_arg_dict : dict
            Dictionnary with arguments that must be given to the plot (used for animated plot)
        save_path : str
            path to the folder where the animations are saved
        logger : logger
            logger used to print path to animation (if None using print instead)
        path_to_image : str
            path to the folder where the image for the animation button is saved
        """
        self.main_widget = main_widget
        # Recovering the object that we want to show and how we want to show it
        self.data = data
        self.param_dict = plot_arg_dict.copy()
        if save_path != "":
            self.save_path = save_path

        self.logger = logger
        self.path_to_image = path_to_image

        # Dynamic import to avoid import loop
        VectorField = import_class("SciDataTool.Classes", "VectorField")

        # Hide or show the comboBox related to the component of a VectorField
        if isinstance(data, VectorField):
            self.data_obj = data  # storing the Vectorfield with all the components while data will only have one component
            self.w_vect_selector.show()
            # Adding/removing axial and comp_z depending on the VectorField object
            self.w_vect_selector.update(self.data_obj)
            self.w_vect_selector.refreshComponent.connect(self.update_component)
            if component is not None:
                self.w_vect_selector.set_component(component)
                self.update_component(is_update_plot=False)
            else:
                self.update_component(is_update_plot=False)
            # Add "all" components in case of quiver plot
            if is_quiver:
                self.w_vect_selector.c_component.blockSignals(True)
                self.w_vect_selector.c_component.insertItem(0, "all")
                self.w_vect_selector.c_component.setCurrentIndex(0)
                self.w_vect_selector.c_component.blockSignals(False)
        else:
            self.w_vect_selector.hide()

        self.w_axis_manager.set_axis_widgets(
            self.data,
            axes_request_list,
            frozen_type,
            is_keep_config=is_keep_config,
            path_to_image=self.path_to_image,
        )
        self.update_range(
            unit=unit,
            z_min=z_min,
            z_max=z_max,
        )

        if is_keep_config:
            self.update_plot()

    def update_component(self, is_update_plot=True):
        """Method that update data according to the component selected in w_vect_selector.
        Parameters
        ----------
        self : WPlotManager
            a WPlotManager object

        """
        component_name = self.w_vect_selector.get_component_selected()

        if component_name in ["radial", "tangential", "axial"]:
            self.data = self.data_obj.to_rphiz().components[component_name]
        elif component_name in ["comp_x", "comp_y", "comp_z"]:
            self.data = self.data_obj.to_xyz().components[component_name]
        elif component_name == "all":
            self.data = self.data_obj.components[
                list(self.data_obj.components.keys())[0]
            ]

        self.w_range.set_range(self.data)

        # Force plot refresh
        if is_update_plot:
            self.update_plot_forced()

    def update_plot(self):
        """Method that update the plot according to the info selected in the UI
        Parameters
        ----------
        self : WPlotManager
            a WPlotManager object

        """
        # Emitting a signal meaning that the plot must be updated
        self.updatePlot.emit()

    def update_plot_forced(self):
        """Method that update the plot according to the info selected in the UI
        Parameters
        ----------
        self : WPlotManager
            a WPlotManager object

        """
        # Emitting a signal meaning that the plot must be updated
        self.updatePlotForced.emit()

    def update_range(self, unit=None, z_min=None, z_max=None):
        """Method that will update the range widget with either the user input or the default value of the DataND object
        Parameters
        ----------
        self : WPlotManager
            a WPlotManager object
        unit : str
            unit in which to plot the field
        z_min : float
            Minimum value for Z axis (or Y if only one axe)
        z_max : float
            Minimum value for Z axis (or Y if only one axe)
        """
        self.w_range.blockSignals(True)

        # Recovering the axis selected and their units
        axes_selected = self.w_axis_manager.get_axes_selected()
        # Recovering the operation on the other axes
        data_selection = self.w_axis_manager.get_operation_selected()

        # Updating the name of the groupBox according to the number of axes selected
        if len(axes_selected) == 1:
            self.w_range.g_range.setTitle("Y Range")
        elif len(axes_selected) == 2:
            self.w_range.g_range.setTitle("Z Range")

        # Setting the WDataRange by sending the necessary info to the widget
        self.w_range.set_range(self.data, unit=unit)

        # If user inputs have been sent (auto plot), then we modify the WDataRange according to these info
        if z_min is not None or z_max is not None:
            self.w_range.set_range_user_input(z_min=z_min, z_max=z_max)

        self.w_range.blockSignals(False)



from SciDataTool.Functions import NormError


def normalize(
    self,
    values,
):
    """Normalizes axis values
    Parameters
    ----------
    self: Norm_vector
        a Norm_vector object
    values: ndarray
        axis values
    Returns
    -------
    Vector of axis values
    """
    if len(values) != len(self.vector):
        raise NormError(
            "Normalization vector is not the same length as axis, cannot normalize"
        )
    else:
        return self.vector



from PySide2.QtWidgets import QWidget

from SciDataTool.GUI.WVectorSelector.Ui_WVectorSelector import Ui_WVectorSelector
from PySide2.QtCore import Signal
from PySide2.QtGui import QStandardItem

COMP_DICT = {
    "all": "all",
    "radial": "radial",
    "circumferential": "tangential",
    "axial": "axial",
    "x-axis component": "comp_x",
    "y-axis component": "comp_y",
    "z-axis component": "comp_z",
}

REV_COMP_DICT = {
    "radial": "radial",
    "tangential": "circumferential",
    "axial": "axial",
    "comp_x": "x-axis component",
    "comp_y": "y-axis component",
    "comp_z": "z-axis component",
}


class WVectorSelector(Ui_WVectorSelector, QWidget):
    """Widget to select how to export the data"""

    refreshComponent = Signal()

    def __init__(self, parent=None):
        """Initialize the UI and linking buttons to their methods

        Parameters
        ----------
        self : WExport
            a WVectorSelector object
        parent : QWidget
            The parent widget
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self, parent=parent)
        self.setupUi(self)

        self.c_component.currentTextChanged.connect(self.update_needed)

        self.c_referential.hide()
        self.in_referential.hide()

        # self.c_referential.currentTextChanged.connect(self.update_needed)

        self.component_selected = None
        self.component_list = list()

    def get_component_selected(self):
        """Getting the component selected

        Parameters
        ----------
        self : WExport
            a WVectorSelector object

        """
        return COMP_DICT[self.c_component.currentText()]

    def set_component(self, component_selected):
        """Method that set the component selected according to the input of the user (auto-plot)
        Parameters
        ----------
        self : DDataPlotter
            a DDataPlotter object
        component_selected : str
            Component to select
        """

        # Setting the combobox with the right component
        if REV_COMP_DICT[component_selected] in self.component_list:
            self.c_component.setCurrentText(REV_COMP_DICT[component_selected])
        else:
            print(
                "WARNING : Trying to set the vector to "
                + component_selected
                + " a component which is not available. Setting to default component"
            )
            self.c_component.setCurrentIndex(1)

    def update(self, data):
        """Updating the combobox according to the components store in the VectorField

        Parameters
        ----------
        self : WExport
            a WVectorSelector object
        data : VectorField
            the object that we want to plot
        """
        comp_stored = data.components.keys()

        self.blockSignals(True)
        self.c_component.clear()
        self.c_component.addItems([REV_COMP_DICT[comp] for comp in comp_stored])
        model = self.c_component.model()
        if "radial" in comp_stored or "tangential" in comp_stored:
            item = QStandardItem("Polar coordinates")
            font = item.font()
            font.setBold(True)
            item.setFont(font)
            item.setEnabled(False)
            model.insertRow(0, item)
            try:
                data.to_xyz()
                item = QStandardItem("Cartesian coordinates")
                font = item.font()
                font.setBold(True)
                item.setFont(font)
                item.setEnabled(False)
                model.insertRow(self.c_component.count(), item)
                self.c_component.addItem("x-axis component")
                self.c_component.addItem("y-axis component")
                if "axial" in comp_stored:
                    self.c_component.addItem("z-axis component")
            except:
                pass
        elif "comp_x" in comp_stored or "comp_y" in comp_stored:
            item = QStandardItem("Cartesian coordinates")
            font = item.font()
            font.setBold(True)
            item.setFont(font)
            item.setEnabled(False)
            model.insertRow(0, item)
            try:
                data.to_rphiz()
                item = QStandardItem("Polar coordinates")
                font = item.font()
                font.setBold(True)
                item.setFont(font)
                item.setEnabled(False)
                model.insertRow(self.c_component.count(), item)
                self.c_component.addItem("radial")
                self.c_component.addItem("circumferential")
                if "comp_z" in comp_stored:
                    self.c_component.addItem("axial")
            except:
                pass

        # Recovering all the components available after the update
        self.component_list = [
            self.c_component.itemText(i)
            for i in range(self.c_component.count())
            if self.c_component.itemText(i)
            not in ["Polar coordinates", "Cartesian coordinates"]
        ]

        # Modifying the width of the dropdown list to make sure that all the element are readable
        component_list = [
            self.c_component.itemText(i) for i in range(self.c_component.count())
        ]
        width_drop_down = max([len(ac) for ac in component_list]) * 6
        self.c_component.view().setMinimumWidth(width_drop_down)

        self.c_component.setCurrentIndex(1)
        self.blockSignals(False)

    def update_needed(self):
        """Emit a signal when the component must be changed

        Parameters
        ----------
        self : WExport
            a WVectorSelector object

        """
        # if self.c_component.currentText() in [
        #     "Polar coordinates",
        #     "Cartesian coordinates",
        # ]:
        #     self.c_component.setCurrentIndex(self.c_component.currentIndex() + 1)

        self.refreshComponent.emit()



# -*- coding: utf-8 -*-
from re import compile, search

from PySide2 import QtGui
from PySide2.QtGui import QDoubleValidator
from PySide2.QtWidgets import QLineEdit

_float_re = compile(r"(([+-]?\d+(\.\d*)?|\.\d+)([eE][+-]?\d+)?)")


def valid_float_string(string):
    """

    Parameters
    ----------
    string :


    Returns
    -------

    """
    match = _float_re.search(string)
    return match.groups()[0] == string if match else False


class FloatEdit(QLineEdit):
    """A Line Edit Widget optimized to input float"""

    def __init__(self, unit="", *args, **kwargs):
        """Same constructor as QLineEdit + config validator"""

        # Call the Line Edit constructor
        super(FloatEdit, self).__init__(*args, **kwargs)

        # Setup the validator
        Validator = FloatValidator(-1e18, 1e18, 10)
        Validator.Notation = QDoubleValidator.ScientificNotation
        self.setValidator(Validator)

    def focusOutEvent(self, e):
        if self.isModified():
            self.editingFinished.emit()
        self.blockSignals(True)
        super(FloatEdit, self).focusOutEvent(e)
        self.blockSignals(False)

    def keyPressEvent(self, event):
        """To send editingFinished when pressing Enter and Return keys"""
        if event.text() == "\r":
            if self.isModified():
                self.editingFinished.emit()
            self.clearFocus()
        else:
            # call base class keyPressEvent
            QLineEdit.keyPressEvent(self, event)

    def setValue(self, value, is_dB=False):
        """Allow to set the containt of the Widget with a float

        Parameters
        ----------
        self :
            A FloatEdit object
        value :
            A float value to set the Text

        Returns
        -------

        """
        if value is None:
            self.clear()
        else:
            if is_dB:
                self.setText(format(value, ".1f"))
            else:
                self.setText(format(value, ".5g"))

    def value(self):
        """Return the content of the Widget as a float

        Parameters
        ----------
        self :
            A FloatEdit object

        Returns
        -------

        """
        try:
            self.setText(self.validator().fixup_txt(self.text()))
            value = float(self.text())
            return value
        except Exception:
            return None


class FloatValidator(QDoubleValidator):
    """DoubleValidator with fixup method to correct the input"""

    def validate(self, string, position):
        """

        Parameters
        ----------
        string :

        position :


        Returns
        -------

        """
        string = string.replace(",", ".")

        match = search(r"^[+-]?($|(\d+\.?|\.?(\d+|$))\d*($|([eE][+-]?)?\d*$))", string)
        is_intermediate = True if match else False

        if valid_float_string(string):
            state = QtGui.QValidator.Acceptable
        elif is_intermediate:
            state = QtGui.QValidator.Intermediate
        else:
            state = QtGui.QValidator.Invalid
        return (state, string, position)

    def fixup_txt(self, text):
        """When the input text is wrong, fixup is called to correct it in
        the field

        Parameters
        ----------
        self :
            A FloatValidator object
        text :
            The text to correct
        gui_unit :
            Current gui unit system
        val_unit :
            Unit used by the FloatEdit

        Returns
        -------

        """
        if text != "":  # We can't correct an empty text
            # 12,10 can't be converted to float 12.10 can
            text = text.replace(",", ".")
            try:
                top = self.top()
                bottom = self.bottom()

                # If the input is too high...
                if float(text) > top:
                    # ... we replace it by the maximum
                    text = str(top)
                # If the input is too low...
                elif float(text) < bottom:
                    # ... we replace it by the minimum
                    text = str(bottom)
            except ValueError:  # Can't convert value to float => not complete
                pass
        return text



import numpy as np
import scipy.integrate as scp_int

from SciDataTool.Functions import AxisError


def antiderivate(values, ax_val, index, Nper, is_aper, is_phys, is_freqs):
    """Returns the anti-derivate of values along given axis
    values is assumed to be periodic and axis is assumed to be a linspace

    Parameters
    ----------
    values: ndarray
        array to derivate
    ax_val: ndarray
        axis values
    index: int
        index of axis along which to derivate
    Nper: int
        number of periods to replicate
    is_aper: bool
        True if values is anti-periodic along axis
    is_phys: bool
        True if physical quantity (time/angle/z)
    is_freqs: bool
        True if frequency axis

    Returns
    -------
    values: ndarray
        anti-derivate of values
    """

    if is_freqs:
        dim_array = np.ones((1, values.ndim), int).ravel()
        dim_array[index] = -1
        axis_reshaped = ax_val.reshape(dim_array)
        values = values / (axis_reshaped * 2 * 1j * np.pi)

    elif is_phys:

        if ax_val.size > 1:
            # Swap axis to always have integration axis on 1st position
            values = np.swapaxes(values, index, 0)
            if Nper is None:
                # Taking input values
                values_full = values
                ax_full = ax_val
            else:
                # Add last point to axis
                ax_full = np.concatenate(
                    (
                        ax_val,
                        np.array([ax_val[-1] + ax_val[1] - ax_val[0]]),
                    )
                )
                # Get values on a full (anti-)period
                shape = list(values.shape)
                shape[0] = shape[0] + 1
                values_full = np.zeros(shape, dtype=values.dtype)
                values_full[:-1, ...] = values
                # Add first sample at the end of values to integrate on last interval
                # Last value is the same as (respectively the opposite of) the first value
                # in case of periodicity (respectively anti-periodicity)
                values_full[-1, ...] = (-1) ** int(is_aper) * values[0, ...]
            # Anti-derivate along axis
            values = np.roll(
                scp_int.cumulative_trapezoid(values_full, x=ax_full, axis=0),
                shift=1,
                axis=0,
            )
            # Integration constant is given by removing average value
            values = values - np.mean(values, axis=0)
            # Get N first values and swap axes back to origin
            values = np.swapaxes(values, 0, index)

        else:
            raise Exception("Cannot anti-derivate along axis if axis size is 1")

    else:
        raise AxisError("Derivation only available for time/angle/z/freqs")

    return values


def integrate_local(values, ax_val, index, Nper, is_aper, is_phys, is_freqs):
    """Returns the local integral of values along given axis (does not change the axis)

    Parameters
    ----------
    values: ndarray
        array to derivate
    ax_val: ndarray
        axis values
    index: int
        index of axis along which to derivate
    Nper: int
        number of periods to replicate
    is_aper: bool
        True if values is anti-periodic along axis
    is_phys: bool
        True if physical quantity (time/angle/z)
    is_freqs: bool
        True if frequency axis

    Returns
    -------
    values_integ: ndarray
        local integration of values
    """

    if not is_phys:
        raise AxisError("Integration only available for time/angle/z")

    if ax_val.size > 1:
        # Compute primitive
        values = antiderivate(values, ax_val, index, Nper, is_aper, is_phys, is_freqs)
        values = np.swapaxes(values, index, 0)
        values_integ = np.zeros(values.shape, dtype=values.dtype)

        # Evaluate the difference between two consecutive primitive values = integral on each segment
        values_diff = values[1:, ...] - values[0:-1, ...]

        # Distribute the integral of each segment on correspond points
        for ival, val in enumerate(values_diff):
            values_integ[ival] += val / 2
            values_integ[ival + 1] += val / 2

        values_integ = np.swapaxes(values_integ, index, 0)

    else:
        raise Exception("Cannot anti-derivate along axis if axis size is 1")

    return values_integ


def integrate_local_pattern(values, ax_val, index):
    """Returns the local integral of values along given axis (does not change the axis)

    Parameters
    ----------
    values: ndarray
        array to derivate
    ax_val: ndarray
        axis values
    index: int
        index of axis along which to derivate
    Nper: int
        number of periods to replicate
    is_aper: bool
        True if values is anti-periodic along axis
    is_phys: bool
        True if physical quantity (time/angle/z)
    is_freqs: bool
        True if frequency axis

    Returns
    -------
    values_integ: ndarray
        local integration of values
    """

    if ax_val.size > 1:
        # Swap axis to always have integration axis on 1st position
        values = np.swapaxes(values, index, 0)

        # Init output arrays
        shape = list(values.shape[1:])
        shape.insert(0, values.shape[0] - 1)
        values_int = np.zeros(shape, dtype=values.dtype)
        ax_int = np.zeros(shape[0])

        # Trapezoidal integration on each segment and calculate position of each segment middle
        for ii in range(shape[0]):
            values_int[ii, ...] = scp_int.trapezoid(
                values[ii : ii + 2, ...], x=ax_val[ii : ii + 2], axis=0
            )
            ax_int[ii] = np.mean(ax_val[ii : ii + 2])

        # Remove zero length interval
        Ia = np.nonzero(np.diff(ax_val))[0]
        values_int = values_int[Ia, ...]
        ax_int = ax_int[Ia]

        values_int = np.swapaxes(values_int, index, 0)

    else:
        raise Exception("Cannot locally integrate along axis if axis size is 1")

    return values_int, ax_int


def integrate(values, ax_val, index, Nper, is_aper, is_phys, is_mean=False):
    """Returns the integral of values along given axis

    Parameters
    ----------
    values: ndarray
        array to integrate
    ax_val: ndarray
        axis values
    index: int
        index of axis along which to derivate
    Nper: int
        number of periodicities along axis
    is_aper: bool
        True if values is anti-periodic along axis
    is_phys: bool
        True if physical quantity (time/angle/z)
    is_mean: bool
        True to divide integrated value by integration interval (mean value)

    Returns
    -------
    values: ndarray
        integration of values
    """

    if not is_phys:
        raise AxisError("Integration only available for time/angle/z")

    if ax_val.size > 1:
        shape = list(values.shape)
        if is_aper:
            # Integration of anti-periodic signal yields zero
            shape0 = [s for ii, s in enumerate(shape) if ii != index]
            values = np.zeros(shape0, dtype=values.dtype)
        else:
            # Swap axis to always have integration axis on 1st position
            values = np.swapaxes(values, index, 0)
            if Nper is None:
                # Taking input values
                values_full = values
                ax_full = ax_val
                Nper = 1
            else:
                # Add last point to axis in case of periodicity
                ax_full = np.concatenate(
                    (
                        ax_val,
                        np.array([ax_val[-1] + ax_val[1] - ax_val[0]]),
                    )
                )
                # Rebuild values on a full (anti-)period
                shape[index], shape[0] = shape[0], shape[index]
                shape[0] = shape[0] + 1
                values_full = np.zeros(shape, dtype=values.dtype)
                values_full[:-1, ...] = values
                # Add first sample at the end of values to integrate on last interval
                # Last value is the same as (respectively the opposite of) the first value
                # in case of periodicity (respectively anti-periodicity)
                values_full[-1, ...] = values[0, ...]
            # Integrate along axis
            values = Nper * np.trapz(values_full, x=ax_full, axis=0)
            # Readd first dim and swap axes back to origin
            values = np.swapaxes(values[None, ...], 0, index)

            if is_mean:
                # Taking mean value by dividing by integration interval
                interval = Nper * (np.nanmax(ax_full) - np.nanmin(ax_full))
                values /= interval

    else:
        raise Exception("Cannot anti-derivate along axis if axis size is 1")

    return values


def derivate(values, ax_val, index, Nper, is_aper, is_phys, is_freqs):
    """Returns the first derivate of values along given axis
    values is assumed to be periodic and axis is assumed to be a linspace

    Parameters
    ----------
    values: ndarray
        array to derivate
    ax_val: ndarray
        axis values
    index: int
        index of axis along which to derivate
    Nper: int
        number of periods to replicate
    is_aper: bool
        True if values is anti-periodic along axis
    is_phys: bool
        True if physical quantity (time/angle/z)
    is_freqs: bool
        True if frequency axis

    Returns
    -------
    values: ndarray
        derivate of values
    """

    if is_freqs:
        dim_array = np.ones((1, values.ndim), int).ravel()
        dim_array[index] = -1
        axis_reshaped = ax_val.reshape(dim_array)
        values = values * axis_reshaped * 2 * 1j * np.pi

    elif is_phys:
        if ax_val.size > 1:
            # Swap axis to always have integration axis on 1st position
            values = np.swapaxes(values, index, 0)
            if Nper is None or is_aper is None:
                # Taking input values
                values_full = values
                ax_full = ax_val
            else:
                # Create the full vector of axis values
                ax_full = np.concatenate(
                    (
                        np.array([ax_val[0] - ax_val[1]]),
                        ax_val,
                        np.array([ax_val[-1] + ax_val[1] - ax_val[0]]),
                    )
                )
                # Rebuild values on a full (anti-)period
                shape = list(values.shape)
                shape[0] = shape[0] + 2
                values_full = np.zeros(shape, dtype=values.dtype)
                values_full[1:-1, ...] = values
                # Add first and last samples at the end and start of values to make values_full periodic
                # Last value is the same as (respectively the opposite of) the first value
                # in case of periodicity (respectively anti-periodicity)
                values_full[-1, ...] = (-1) ** int(is_aper) * values[0, ...]
                values_full[0, ...] = (-1) ** int(is_aper) * values[-1, ...]
            # Derivate along axis
            values = np.gradient(values_full, ax_full, axis=0)
            # Get N first values and swap axes back to origin
            values = np.swapaxes(values[1:-1, ...], 0, index)

        else:
            raise Exception("Cannot derivate along axis if axis size is 1")
    else:
        raise AxisError("Derivation only available for time/angle/z/freqs")

    return values



from numpy import allclose, linspace


def to_linspace(self):
    """Tests if values are a linspace and returns a DataLinspace (or Data1D if not possible)
    Parameters
    ----------
    self: Data1D
        a Data1D object
    Returns
    -------
    DataLinspace or Data1D
    """

    # Dynamic import to avoid loop
    module = __import__("SciDataTool.Classes.DataLinspace", fromlist=["DataLinspace"])
    DataLinspace = getattr(module, "DataLinspace")

    values = self.values
    number = values.size

    if (
        number > 1
        and values.dtype.kind not in {"U", "S"}
        and allclose(
            values,
            linspace(values[0], values[-1], number, endpoint=True),
            rtol=1e-5,
            atol=1e-8,
            equal_nan=False,
        )
    ):
        New_axis = DataLinspace(
            initial=values[0],
            final=values[-1],
            number=number,
            include_endpoint=True,
            name=self.name,
            unit=self.unit,
            symmetries=self.symmetries,
            normalizations=self.normalizations.copy(),
            is_components=self.is_components,
            symbol=self.symbol,
        )
    else:
        New_axis = self.copy()

    return New_axis



def to_datadual(self, datatime=None):
    """Creates a DataDual object.
    Parameters
    ----------
    self : DataFreq
        a DataFreq object
    datatime : DataTime
        a DataTime object
    Returns
    -------
    a DataDual object
    """

    # Dynamic import to avoid loop
    module = __import__("SciDataTool.Classes.DataDual", fromlist=["DataDual"])
    DataDual = getattr(module, "DataDual")

    # Fill DataDual object with fourier data
    datadual = DataDual(
        name=self.name,
        symbol=self.symbol,
        unit=self.unit,
        normalizations=self.normalizations.copy(),
        axes_df=self.axes,
        values_df=self.values,
    )

    # Add time/space data (from input or using ifft)
    if datatime is None:
        datatime = self.freq_to_time()
    datadual.axes_dt = datatime.axes
    datadual.values_dt = datatime.values

    return datadual
